################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
genelist_analysis.py -analyse gene lists (GO, etc.)
===================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse

import CGAT.Experiment as E
import CGAT.GO as GO
import CGAT.IOTools as IOTools
import CGAT.Stats as Stats

def computeFDR( all_results, 
                qvalue_method = "storey" ):
    '''compute FDR.
    
    update GOResult structure with field .fdr
    '''
    
    # flatten all_results
    results = []
    for key, data in all_results.iteritems():
        results.extend( data.mResults.values() )
    
    observed_min_pvalues = [ min(x.mProbabilityOverRepresentation,
                                 x.mProbabilityUnderRepresentation) for x in results ]
    
    if qvalue_method == "storey":

        # compute fdr via Storey's method
        fdr_data = Stats.doFDR( observed_min_pvalues, vlambda = 0.1)

        E.info( "estimated proportion of true null hypotheses = %6.4f" % fdr_data.mPi0 ) 

        if fdr_data.mPi0 < 0.1:
            E.warn( "estimated proportion of true null hypotheses is less than 10%%  (%6.4f)" % fdr_data.mPi0) 
            
        for result, qvalue in zip( results, fdr_data.mQValues ):
            result.fdr = qvalue

    elif options.qvalue_method == "empirical":
        assert options.sample > 0, "requiring a sample size of > 0"
        raise NotImplementedError("empirical needs some work" )

def outputOntologyResults( all_results, go2infos, options ):
    '''output ontology results

    The following tables will be created:
    tests - overview of individual gene list results
    all - full output
    '''
    
    headers = ["ontology",
               "genelist",
               "code" ] +\
               GO.GOResult().getHeaders() +\
               GO.GOInfo().getHeaders()

    if options.fdr: 
        has_fdr = True
        headers += ["fdr"]

    outfile = options.stdout

    outfile.write("\t".join(headers) + "\n" )

    for testpair, data in all_results.iteritems():
        ontology, genelist = testpair
        for key, result in data.mResults.iteritems():
            code = GO.GetCode( result )

            n = go2infos[ontology].get( key, GO.GOInfo() )
            outfile.write( "\t".join( (ontology, genelist, code, str(result), str(n))) )
            if has_fdr: outfile.write("\t%e" % result.fdr)
            outfile.write("\n")

    
def doOntologyAnalysis( gene_lists, options ):
    '''do ontology analysis - requires options.filename_assignments to be set.'''
    
    
    E.info( "reading association of categories and genes from %s" % (options.filename_assignments) )
    gene2gos, go2infos = GO.ReadGene2GOFromFile( IOTools.openFile(options.filename_assignments ) )
    E.info( "read %i ontologies" % (len(gene2gos)) )

    #############################################################
    ## sort out which ontologies to test
    ontologies = options.ontology

    # test all if none specified
    if not ontologies: ontologies = gene2gos.keys()

    all_results = {}

    for ontology in ontologies:
        gene2go, go2info = gene2gos[ontology], go2infos[ontology]

        if len(go2info) == 0:
            E.warn( "could not find information for terms - could be mismatch between ontologies")

        ngenes, ncategories, nmaps = GO.CountGO( gene2go )        
        E.info( "%s: ontology assignments: %i genes mapped to %i categories (%i maps)" % 
                (ontology, ngenes, ncategories, nmaps) )

        for geneset, x in gene_lists.iteritems():
            foreground, background = x
            
            E.debug( "working on %s - %s" % (ontology, geneset))

            E.info( "%s - %s: (unfiltered) foreground=%i, background=%i" % (ontology, 
                                                                            geneset, 
                                                                            len(foreground), 
                                                                            len(background)))
            
            results = GO.AnalyseGO( gene2go, foreground, background )

            if len(results.mSampleGenes) == 0:
                E.warn( "%s - %s: no genes with GO categories - analysis aborted" % (ontology, geneset ) )
                continue

            # add sampling at this point for empirical FDR
            
            all_results[ (ontology,geneset) ] = results
    
    if options.fdr:
        E.info("computing the FDR with method %s" % options.qvalue_method )
        computeFDR( all_results, 
                    qvalue_method = options.qvalue_method )
    
    outputOntologyResults( all_results,
                           go2infos,
                           options )

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    parser = E.OptionParser( version = "%prog version: $Id: GO.py 2883 2010-04-07 08:46:22Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-s", "--species", dest="species", type="string",
                      help="species to use [default=%default]." )

    parser.add_option("-i", "--slims", dest="filename_slims", type="string",
                      help="filename with GO SLIM categories [default=%default].")

    parser.add_option( "-g", "--genes", dest="filename_genes", type="string",
                       help="filename with genes to analyse [default=%default]." )

    parser.add_option( "-f", "--format", dest="filename_format", type="choice",
                       choices = ("list", "matrix" ),
                       help="filename format [default=%default]." )

    parser.add_option( "-b", "--background", dest="filename_background", type="string",
                       help="filename with background genes to analyse [default=%default]." )

    parser.add_option( "-o", "--sort-order", dest="sort_order", type="choice",
                       choices=("fdr", "pover", "ratio" ),
                       help="output sort order [default=%default]." )

    parser.add_option( "--ontology", dest="ontology", type="choice", action="append",
                       choices=("biol_process","cell_location","mol_function", "mgi" ),
                       help="go ontologies to analyze. Ontologies are tested separately."
                       " [default=%default]." )

    parser.add_option( "-t", "--threshold", dest="threshold", type="float",
                       help="significance threshold [>1.0 = all ]. If --fdr is set, this refers to the fdr, otherwise it is a cutoff for p-values." )

    parser.add_option ("--filename-dump", dest="filename_dump", type="string",
                       help="dump GO category assignments into a flatfile [default=%default]." )

    parser.add_option ("--filename-ontology", dest="filename_ontology", type="string",
                       help="filename with ontology in OBO format [default=%default]." )

    parser.add_option ( "--filename-assignments", dest="filename_assignments", type="string",
                        help="read ontology assignments from a flatfile [default=%default]." )

    parser.add_option ( "--sample", dest="sample", type="int",
                        help="do sampling (with # samples) [default=%default]." )
    
    parser.add_option ( "--filename-output-pattern", dest = "output_filename_pattern", type="string",
                        help="pattern with output filename pattern (should contain: %(go)s and %(section)s ) [default=%default]")

    parser.add_option ( "--output-filename-pattern", dest = "output_filename_pattern", type="string",
                        help="pattern with output filename pattern (should contain: %(go)s and %(section)s ) [default=%default]")
    
    parser.add_option ( "--fdr", dest="fdr", action="store_true",
                       help="calculate and filter by FDR [default=%default]." )

    parser.add_option ( "--go2goslim", dest="go2goslim", action="store_true",
                       help="convert go assignments in STDIN to goslim assignments and write to STDOUT [default=%default]." )

    parser.add_option ( "--gene-pattern", dest = "gene_pattern", type="string",
                        help="pattern to transform identifiers to GO gene names [default=%default].")

    parser.add_option( "--filename-map-slims", dest="filename_map_slims", type="string",
                       help="write mapping between GO categories and GOSlims [default=%default].")

    parser.add_option( "--get-genes", dest="get_genes", type="string",
                       help="list all genes in the with a certain GOID [default=%default]." )

    parser.add_option( "--strict", dest="strict", action="store_true",
                       help="require all genes in foreground to be part of background. "
                       "If not set, genes in foreground will be added to the background [default=%default]." )

    parser.add_option("-q", "--qvalue-method", dest="qvalue_method", type="choice",
                      choices = ( "empirical", "storey" ),
                      help="method to perform multiple testing correction by controlling the fdr [default=%default]."  )

    parser.add_option( "--qvalue-lambda", dest="qvalue_lambda", type="float",
                       help="fdr computation: set lambda to fixed value [default=%default]."  )

    # parser.add_option( "--qvalue-pi0-method", dest="qvalue_pi0_method", type="choice",
    #                    choices = ("smoother", "bootstrap" ),
    #                    help="fdr computation: method for estimating pi0 [default=%default]."  )
    
    parser.set_defaults( species = None,
                         filename_genes = "-",
                         filename_format = "list",
                         filename_ontology = None,
                         filename_assignments = None,
                         filename_background = None,
                         filename_slims = None,
                         ontology = [],
                         filename_dump = None,
                         sample = 0,
                         fdr = False,
                         output_filename_pattern = None,
                         threshold = 0.05,
                         filename_map_slims = None,
                         gene_pattern = None,
                         sort_order = "ratio",
                         get_genes = None,
                         strict = False,
                         qvalue_method = "empirical",
                         qvalue_lambda = None,
                         )

    (options, args) = E.Start( parser, argv = argv )

    # collect all gene lists
    # gene lists are tuples of fg/bg.
    # If bg is None, the full list of genes is 
    # taken as the background.
    gene_lists = {}
    bg = None

    if options.filename_format == "list":
        gene_lists[ "default"] = (GO.ReadGeneList( options.filename_genes, 
                                                   gene_pattern = options.gene_pattern ),
                                  None )

    elif options.filename_format == "matrix":
        bg, genes = GO.ReadGeneLists( options.filename_genes,
                                  gene_pattern = options.gene_pattern ) 

        # use default background        
        gene_lists = dict( [ (x, (y,None)) for x,y in genes.iteritems() ] )

    if bg: E.info("gene list: %20s = %i genes" % ("default background", len(bg)))

    E.info( "read %i gene list from %s" % (len(gene_lists), options.filename_genes) )
    if len(gene_lists) == 0:
        raise ValueError( "no gene lists input")

    # set the default background for those tests without explicit background
    for key in gene_lists.keys():
        if gene_lists[key][1] == None:
            gene_lists[key] = (gene_lists[key][0], bg )

    # check for inconsistencies
    for key, l in gene_lists.iteritems():
        foreground, background = l

        missing = set(foreground).difference( set(background))

        if options.strict:
            assert len(missing) == 0, \
                "%s: %i genes in foreground but not in background: %s" % (key, len(missing), str(missing))
        else:
            if len(missing) != 0:
                E.warn( "%s: %i genes in foreground that are not in background - added to background of %i" %\
                            (key, len(missing), len(background)) )
            background.extend( missing )


    for key, l in gene_lists.iteritems():
        lfg = len(l[0])
        lbg =len(l[1])

        E.debug( "gene list: %20s = %i genes / %i background = %5.2f %%" % 
                 (key, lfg, lbg, 100.0 * lfg / lbg ) )


    ## read gene list assignments
    if options.filename_assignments:
        doOntologyAnalysis( gene_lists, options )


    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
extractseq.py - extract sequences/sequence regions from a fasta file
====================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python extractseq.py --help

Type::

   python extractseq.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import optparse
import math

import CGAT.Experiment as E
import CGAT.IndexedFasta as IndexedFasta

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: extractseq.py 2861 2010-02-23 17:36:32Z andreas $")

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="pattern to look for sequence filename."  )

    parser.add_option("-d", "--identifier", dest="identifier", type="string",
                      help="identifier(s)."  )

    parser.add_option("-o", "--output-coordinate-format", dest="output_coordinate_format", type="choice",
                      choices=("full", "long" ),
                      help="""output format of coordinates. Output format is contig:strand:from:to in zero based
/forward/reverse strand coordinates in open/closed notation. 'long' includes the contig length as fifth field"""  )

    parser.add_option( "--input-format", dest="input_format", type="choice",
                       choices=("list", "id-compressed" ),
                       help="input format." )

    parser.add_option( "-i", "--input-coordinate-format", dest="input_coordinate_format", type="choice",
                       choices=( "zero-both", "zero-forward" ),
                       help="coordinate format." )

    parser.add_option("-e", "--extend-region", dest="extend_region", type="int",
                      help="regions are extended by this margin at either end."  )

    parser.add_option("-r", "--shorten-region", dest="shorten_region", type="int",
                      help="regions are shortened by this margin at either end."  )

    parser.set_defaults(
        genome_file = None,
        identifier=None,
        input_coordinate_format= "zero-both",
        output_coordinate_format = "full",
        input_format = "list",
        extend_region = 0,
        shorten_region = 0,
        )

    (options, args) = E.Start( parser )

    fasta = IndexedFasta.IndexedFasta( options.genome_file )

    lines = []
    if options.identifier:
        lines += map(lambda x: x.split(":"), options.identifier.split(","))

    if args:
        lines += map(lambda x: x.split(":"), args)        
    
    if len(lines) == 0:
        lines = map( lambda x: x[:-1].split("\t"), (filter(lambda x: x[0] != "#", options.stdin.readlines())))

    ninput, nskipped, noutput = 0, 0, 0
    for data in lines:

        if options.input_format == "list":
            if len(data) < 4:
                sbjct_token = data[0]
                sbjct_from, sbjct_to = "0", "0"
                sbjct_strand = "+"
            else:
                sbjct_token, sbjct_strand, sbjct_from, sbjct_to = data[:4]
            id = None
        elif options.input_format == "id-compressed":
            id = data[0]
            sbjct_token, sbjct_strand, sbjct_from, sbjct_to = data[1].split(":")

        ninput += 1

        try:
            sbjct_from, sbjct_to = int(sbjct_from), int(sbjct_to)
        except ValueError:
            E.warn("skipping line %s" % data)
            nskipped += 1
            continue
        
        sbjct_from -= (options.extend_region - options.shorten_region)
        sbjct_from = max( 0, sbjct_from )
        lcontig = fasta.getLength( sbjct_token ) 
        if sbjct_to != 0:
            sbjct_to   += (options.extend_region - options.shorten_region)
            sbjct_to = min(sbjct_to, lcontig )
        else:
            sbjct_to = lcontig

        if sbjct_to - sbjct_from <= 0:
            nskipped += 1
            continue

        sequence = fasta.getSequence( sbjct_token, sbjct_strand,
                                      sbjct_from, sbjct_to,
                                      converter = IndexedFasta.getConverter( options.input_coordinate_format ) )

        if options.output_coordinate_format == "full":
            coordinates = "%s:%s:%i:%i" % (sbjct_token,
                                           sbjct_strand,
                                           sbjct_from,
                                           sbjct_to)
            
        elif options.output_coordinate_format == "long":
            coordinates = "%s:%s:%i:%i:%i" % (sbjct_token,
                                              sbjct_strand,
                                              sbjct_from,
                                              sbjct_to,
                                              lcontig )
            
        if id:
            options.stdout.write( ">%s %s\n%s\n" % (id, coordinates, sequence ) )            
        else:
            options.stdout.write( ">%s\n%s\n" % (coordinates, sequence ) )

        noutput += 1

    E.info("ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput, nskipped))

    E.Stop()



################################################################################
#   Gene prediction pipeline 
#
#   $Id: AGP.py 2781 2009-09-10 11:33:14Z andreas $
#
#   Copyright (C) 2005 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
AGP.py - working with AGP files 
=====================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

to assemble contigs to scaffolds.

Code
----

"""

class ObjectPosition:
    def __init__(self):
        pass

    def map( self, start, end ):
        if self.mOrientation:
            return start + self.start, end + self.start
        else:
            return end + self.start, start + self.start
        
class ComponentPosition:
    def __init__(self):    
        pass


class AGP:
    
    ##------------------------------------------------------------
    def readFromFile( self, infile ):
        """read an agp file.

        Example line:

        scaffold_1      1       1199    1       W       contig_13       1       1199    +

        converts coordinates to zero-based coordinates using open/closed notation.
        In AGP nomenclature
        (http://www.ncbi.nlm.nih.gov/genome/guide/Assembly/AGP_Specification.html)
        objects (obj) like scaffolds are assembled from components (com) like contigs.

        Component types are:
        W: WGS sequence
        N: gap of specified length
        """

        self.mMapComponent2Object = {}
        self.mMapObject2Component = {}

        for line in infile:
            if line[0] == "#": continue

            data = line[:-1].split("\t")

            obj_id, obj_start, obj_end, ncoms, com_type, com_id = data[:6]

            if com_type == "N": continue                
            com_start, com_end, orientation = data[6:9]

            obj_start, obj_end = int(obj_start)-1, int(obj_end)
            com_start, com_end = int(com_start)-1, int(com_end)        

            orientation = orientation in ("+", "0", "na" )
            
            if com_start != 0:
                raise "beware, non zero com_start"

            object = ObjectPosition()
            object.mId = obj_id
            object.start = obj_start
            object.end = obj_end
            object.mOrientation = orientation
            
            self.mMapComponent2Object[com_id] = object 

    def mapLocation( self, id, start, end ):
        """map a genomic location."""

        if id not in self.mMapComponent2Object:
            raise KeyError, "id %s is not known" % (id)

        pos = self.mMapComponent2Object[id]
        return (pos.mId, ) + pos.map( start, end )

        

        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
Synteny.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import string, re, types
### Tools to analyze synteny
import Genomics

###################################################################
        
class Ortholog:
    def __init__(self, ortholog_id, gene_id, contig, strand, first_res, last_res):
        (self.mOrthologId, self.gene_id, self.contig, self.strand,
         self.mFrom, self.mTo) = ortholog_id, gene_id, contig, strand, first_res, last_res
        self.mRank = 0

    def __str__(self):
        return ";".join(map(str, ( self.mOrthologId, self.gene_id, self.contig, self.strand,
                                   self.mFrom, self.mTo, self.mRank)) )
    
###################################################################
def ReadOrthologs( infile ):
    """read orthologs from file."""
    orthologs = {}
    
    for line in infile:
        if line[0] == "#": continue

        ortholog_id, gene_id, contig, strand, first_res, last_res = line[:-1].split("\t")[:6]

        if ortholog_id not in orthologs:
            orthologs[ortholog_id] = []

        orthologs[ortholog_id].append( Ortholog(ortholog_id,
                                                gene_id, contig, strand,
                                                int(first_res), int(last_res)) )
        
    return orthologs

###################################################################
def ReadOrthologsPerSpecies( infile ):
    """read orthologs from file with species information."""
    orthologs = {}

    read = set()
    
    for line in infile:
        
        if type(line) in (types.TupleType, types.ListType):
            ortholog_id, schema, gene_id, contig, strand, first_res, last_res = line
        else:
            if line[0] == "#": continue
            ortholog_id, schema, gene_id, contig, strand, first_res, last_res = line[:-1].split("\t")[:7]

        key = "%s-%s" % (schema,gene_id)
        if key in read: continue
        read.add(key)

        if schema not in orthologs:
            orthologs[schema] = {}

        o = orthologs[schema]
        if ortholog_id not in o:
            o[ortholog_id] = []

            
        o[ortholog_id].append( Ortholog(ortholog_id,
                                        gene_id, contig, strand,
                                        int(first_res), int(last_res)) )
        
    return orthologs

###################################################################
def FilterJunk( orthologs ):
    """remove assignments to junk contigs.
    """

    for id, oo in orthologs.items():
        oo = filter( lambda x: not Genomics.IsJunk( x.contig ), oo )
        if len(oo) > 0:
            orthologs[id] = oo
        else:
            del orthologs[id]

###################################################################
def CleanOrthologs( orthologs1, orthologs2, filter_junk = False ):
    """only keep orhtologs that are present in both sets."""

    if filter_junk:
        FilterJunk( orthologs1 )
        FilterJunk( orthologs2 )        

    for id, oo in orthologs1.items():
        if id not in orthologs2:
            del orthologs1[id]

    for id, oo in orthologs2.items():
        if id not in orthologs1:
            del orthologs2[id]

###################################################################
def GetContigs( orthologs ):
    """get map of contigs to orthologs.

    An ortholog can be part of only one contig, but the same ortholog_id can
    be part of several contigs.
    """
    
    contigs = {}
    for id, oo in orthologs.items():
        for o in oo:
            if o.contig not in contigs: contigs[o.contig] = []
            contigs[o.contig].append( o )
            
    return contigs

###################################################################
def GetMap( values ):
    map_a2b = {}
    map_b2a = []

    for v in values:
        map_a2b[v] = len(map_b2a)
        map_b2a.append(v)

    return map_a2b, map_b2a

###################################################################
def SetRankToPosition( orthologs, map_contig2orthologs, relative = True ):
    """set rank of ortholog to position within contig."""

    rank = 0    
    for contig, oo in map_contig2orthologs.items():
        oo.sort( lambda x,y: cmp( x.mFrom, y.mFrom) )
        if relative: rank = 0
        for o in oo:
            o.mRank = rank
            rank += 1

###################################################################
def GetContigCounts( orthologs1, orthologs2 ):
    """count number of orthologs per contig pair.
    """
    counts = {}
    
    for id, oo1 in orthologs1.items():
        
        if id not in orthologs2: continue
            
        conts1 = map( lambda x: x.contig, oo1 )
        conts2 = map( lambda x: x.contig, orthologs2[id] )

        for c1 in conts1:
            for c2 in conts2:
                key = (c1, c2)
                if key not in counts: counts[key] = 0
                counts[key] += 1
                
    return counts

###################################################################
def GetContigContents( orthologs1, orthologs2 ):
    """count number of orthologs per contig pair.
    """
    counts = {}
    
    for id, oo1 in orthologs1.items():
        
        if id not in orthologs2: continue
            
        conts1 = map( lambda x: x.contig, oo1 )
        conts2 = map( lambda x: x.contig, orthologs2[id] )

        for c1 in conts1:
            for c2 in conts2:
                key = (c1, c2)
                if key not in counts: counts[key] = []
                counts[key].append( id )
                
    return counts

###################################################################
def SortAssignOrthologs( orthologs1, orthologs2 ):
    """based on the number of shared orthologs, sort the contigs in
    both ortholog sets."""

    contigs1 = GetContigs( orthologs1 )
    contigs2 = GetContigs( orthologs2 )    

    ## Full matrix too large, when I have many contigs.
    ## Thus I use a sparse matrix and do some straightforward
    ## sorting
    counts = GetContigCounts( orthologs1, orthologs2 )

    llinks = []
    for key in counts:
        llinks.append( (counts[key], key) )

    llinks.sort()
    llinks.reverse()

    sorted_list1 = []
    sorted_list2 = []

    sorted1 = {}
    sorted2 = {}

    for n, key in llinks:
        
        ## do not take empty contigs
        if n == 0: continue
        
        a,b = key

        if a not in sorted1:
            sorted1[a] = True
            sorted_list1.append(a)
            
        if b not in sorted2:
            sorted2[b] = True
            sorted_list2.append(b)
            
    return sorted_list1, sorted_list2, contigs1, contigs2

###################################################################
def GetSortedOrthologs( orthologs ):
    """sort orthologs by contig and rank."""

    sorted_orthologs = []
    
    for ortholog_id, oo in orthologs.items():
        sorted_orthologs += oo
        
    sorted_orthologs.sort( lambda x,y: cmp(x.mRank, y.mRank) )
    
    return sorted_orthologs

###################################################################
class Block:
    def __init__(self):
        self.mMembers1 = []
        self.mMembers2 = []        
        self.mBlockId = 0
        self.mBreakers1 = []
        self.mBreakers2 = []        
        
    def __str__(self):
        return "%i\t%i\t%s\t%i\t%i\t%i\t%i\t%i\t%s\t%i\t%i\t%i\t%i" % (self.mBlockId,
                                                               len(self.mMembers1),
                                                               self.contig1,
                                                               self.mFrom1,
                                                               self.mTo1,
                                                               self.mTo1 - self.mFrom1,
                                                               len(self.mBreakers1),                                                               
                                                               len(self.mMembers2),
                                                               self.contig2,
                                                               self.mFrom2,
                                                               self.mTo2,
                                                               self.mTo2 - self.mFrom2,
                                                               len(self.mBreakers2))
    def getHeader( self ):
        return "\t".join(("blockid", "nmem1", "contig1", "from1", "to1", "len1", "nbreaks1",
                         "nmem2", "contig2", "from2", "to2", "len2", "nbreaks2"))

    def cleanMembers( self, members ):
        """remove duplicates from members list."""
        taken = {}
        new_members = []
        for o in members:
            if o.mRank in taken: continue
            
            new_members.append( o )
            taken[o.mRank] = True
                
        return list( new_members )
        
    def cleanBreakers( self, members, breakers ):
        """remove small inversion from the list of synteny breakers."""
        n2 = set( map(lambda x: x.mRank, members) )
        taken = {}
        new_members = []
        for o in breakers:
            if o.mRank in taken: continue
            
            if o.mRank not in n2:
                new_members.append( o )
                taken[o.mRank] = True
                
        return list( new_members )
            
    def update( self ):
        if self.mMembers1:
            self.mMembers1 = self.cleanMembers( self.mMembers1 )
            self.contig1 = self.mMembers1[0].contig
            self.mFrom1 = min(map(lambda x: x.mFrom, self.mMembers1) )
            self.mTo1 = max(map(lambda x: x.mTo, self.mMembers1) )
            self.mBreakers1 = self.cleanBreakers( self.mMembers1, self.mBreakers1 )            
        if self.mMembers2:
            self.mMembers2 = self.cleanMembers( self.mMembers2 )            
            self.contig2 = self.mMembers2[0].contig
            self.mFrom2 = min(map(lambda x: x.mFrom, self.mMembers2) )
            self.mTo2 = max(map(lambda x: x.mTo, self.mMembers2) )
            self.mBreakers2 = self.cleanBreakers( self.mMembers2, self.mBreakers2 )
            
###################################################################
"""The syntenic relationships of the D. melanogaster protein to D. pseudoobscura sequence anchor points described by these BLAST
hits were then manually inspected to define and refine the order of syntenic blocks. Synteny blocks were defined as runs of consecutive
D. melanogaster protein sequence?D. pseudoobscura genomic sequence pairs. Within a syntenic block, gaps were permitted (since there are
genes that fall into sequence gaps) and an occasional gene out of order was also permitted (if it fell within five genes of its expected location).
Gene duplications in one species could be perceived as synteny breaks. Gene duplications were not considered in the derivation of synteny blocks.

Figure 1 shows the synteny blocks of the chromosomes are short and extremely mixed, but the great majority of syntenic sequences are
found on the same Muller element in D. pseudoobscura as they are in D. melanogaster. Thus, as expected, the majority of the chromosomal
rearrangements between the D. pseudoobscura and D. melanogaster lineages have been confined to related chromosome arms. The average number
of D. melanogaster genes in a syntenic block is 10.7, corresponding to ~83 kb. The length distribution of syntenic blocks on different
Muller elements is shown in Supplemental Figure S2.
"""

def GetSyntenicOrthologs( orthologs, last_contig, last_rank,
                          max_synteny_distance = 5 ):
    """get orthologs that are syntenic to previous one."""
    oo = []
    for o in orthologs:
        if o.contig == last_contig and \
               abs(o.mRank - last_rank) <= max_synteny_distance:
            oo.append(o)
    return oo

def GetSyntenyBlocks( orthologs1, orthologs2,
                      max_synteny_distance = 5,
                      max_look_ahead = 1,
                      loglevel = 0 ):
    """compute synteny blocks between orthologs.

    max_synteny_distance: number of rearrangements and deletions that are to be
    tolerated.

    This is the distance between adjacent orthologs, minimum should be 1.
    The ones causeing the rearrangement/deletion are saved in
    block.mRearrangements.

    max_look_ahead: if there is a synteny break, look ahead these number of
    orthologs in species 1 to see if the block continues.
    """

    ## sort orthologs1 by contig and rank and group into
    ## micro-synteny blocks

    sorted_orthologs = GetSortedOrthologs( orthologs1 )
    sorted_orthologs2 = GetSortedOrthologs( orthologs2 )    

    blocks = []

    last_contig1 = None
    last_contig2 = None

    last_rank2 = None

    block = None

    no1 = 0
    while no1 < len(sorted_orthologs):

        o1 = sorted_orthologs[no1]
        test_oo2 = orthologs2[o1.mOrthologId]
        
        if loglevel >= 2:
            print "# last=", last_contig1, last_contig2, last_rank2
            print "# o1=", str(o1)
            print "# o2=", map(str, orthologs2[o1.mOrthologId])
        
        # find the syntenic ones within the other orthologs
        # In case of local duplication, this might be several
        found_oo2 = GetSyntenicOrthologs( test_oo2,
                                          last_contig2, last_rank2,
                                          max_synteny_distance )
        ## if no match has been found
        if o1.contig != last_contig1 or \
               len(found_oo2) == 0:

            ## check for lookahead:
            for x in range(no1 + 1, min(no1 + 1 + max_look_ahead, len(sorted_orthologs) )):
                
                new_oo2 = GetSyntenicOrthologs( orthologs2[sorted_orthologs[x].mOrthologId],
                                                last_contig2, last_rank2,
                                                max_synteny_distance )
                ## if something is found
                ## save orthologs causing the breaks
                ## advance index
                if len(new_oo2) > 0:
                    for o2 in test_oo2:
                        if loglevel >= 3:
                            print "## added breaker to 2: %s" % str(o2)
                        block.mBreakers2.append( o2 )
                    no1 = x - 1
                    found_oo2 = new_oo2
                    last_rank2 = found_oo2[0].mRank                    
                    break
                
            else:
                ## if lookahead does not continue break, continue
                if block:
                    if loglevel >= 2:
                        print "############saving cluster %i#####################################" % block.mBlockId
                    
                    block.update()
                    blocks.append( block )

                block = Block()
                block.mBlockId = len(blocks)
                    
                found_oo2 = orthologs2[o1.mOrthologId]
                last_rank2 = found_oo2[0].mRank
        else:
            ## save rank for synteny breakers that are skipped over.
            ## find minimum distance to last_rank2
            min_rank = min(map(lambda x: x.mRank, found_oo2 ))
            max_rank = max(map(lambda x: x.mRank, found_oo2 ))

            if last_rank2 < min_rank:
                for x in range( last_rank2 + 1, min_rank):
                    if loglevel >= 3:
                        print "# added skippers to 2: ", sorted_orthologs2[x]
                    block.mBreakers2.append( sorted_orthologs2[x] )
                last_rank2 = max_rank
            elif last_rank2 > max_rank:
                for x in range( max_rank+1, last_rank2):
                    if loglevel >= 3:
                        print "# added skippers to 2: ", sorted_orthologs2[x]
                    block.mBreakers2.append( sorted_orthologs2[x] )
                last_rank2 = min_rank
            else:
                ## do nothing, if rank within middle (happens in m:m clusters)
                pass

        block.mMembers1.append( o1 )
        for o2 in found_oo2:
            block.mMembers2.append( o2 )        
            
        last_contig1 = o1.contig
        last_contig2 = o2.contig
        no1 += 1

    return blocks

##--------------------------------------------------------------------------------
def Blocks2Orthologs( blocks ):
    """from a list of blocks, return two lists of orthologs.
    """
    orthologs1 = []
    orthologs2 = []
    
    for block in blocks:
        o1 = Ortholog( block.mBlockId, block.mBlockId, block.contig1, "+", block.mFrom1, block.mTo1)
        orthologs1.append(o1)
        o2 = Ortholog( block.mBlockId, block.mBlockId, block.contig2, "+", block.mFrom2, block.mTo2)
        orthologs2.append(o2)

    return orthologs1, orthologs2




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
csv_uniq.py - make rows in table uniq
=====================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python csv_uniq.py --help

Type::

   python csv_uniq.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

import CGAT.Experiment as E
import csv



def ConvertDictionary( d ):
    """tries to convert values in a dictionary.
    """

    rx_int = re.compile("^[+-]*[0-9]+$")
    rx_float = re.compile("^[+-]*[0-9.+\-eE]+$")
    for k,v in d.items():
        if rx_int.match( v ):
            d[k] = int(v)
        elif rx_float.match( v ):
            d[k] = float(v)

    return d
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: csv_uniq.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.set_defaults(
        remove = False,
        unique = False,
        )

    (options, args) = E.Start( parser, add_csv_options  = True)

    input_fields = args

    lines = filter( lambda x: x[0] != "#", sys.stdin.readlines())

    if len(lines) > 0:
        old_fields = lines[0][:-1].split("\t")

        reader = csv.DictReader( lines,
                                 dialect=options.csv_dialect )

        writer = csv.DictWriter( sys.stdout,
                                 fields,
                                 dialect=options.csv_dialect,
                                 lineterminator = options.csv_lineterminator,
                                 extrasaction = 'ignore' )
        
        print "\t".join(fields)        

        first_row = True
        for row in reader:
            row = ConvertDictionary( row )
            writer.writerow(row)

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fastq2N.py -
=============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   This script takes as input a fastq file and converts a position (base) to an N
   call. This is to keep all reads in line should there be an overrepresentation
   of N calls in any samples.

Usage
-----

Example::

   python fastq2N.py --help

Type::

   python fastq2N.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import CGAT.Experiment as E
import CGAT.Fastq as Fastq
import gzip

# define functions to be used in main
def replace(fastqfile, baseToReplace):
    '''replaces the specified base with N'''

    # use gzip as default to open the fastq file
    outf = gzip.open("replaced_" + fastqfile, "w")
    fastq = gzip.open(fastqfile)
    iterator = Fastq.iterate(fastq)
    for record in iterator:
        x = list(record.seq)
        x[int(baseToReplace)] = "N"
        record.seq = "".join(x)
        outf.write("@" + record.identifier + "\n" + record.seq + "\n" + "+" + record.identifier + "\n" + record.quals + "\n")
        
        

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $",
                             usage = globals()["__doc__"] )

    parser.add_option("-i", "--infile", dest="infile", type="string",
                      help="Input filename")
    parser.add_option("-b", "--base-position", dest = "base_position",
                      help = "base position in which to replace with an N")

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if not options.infile:
        raise ValueError ("need to specify input fastq file")

    if not options.base_position:
        raise ValueError ("need to specify which base to convert")
    
    # main function
    replace(options.infile, options.base_position)
    
    ## write footer and output benchmark information.
    E.Stop()
    

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
data2bins.py - split table according to values in a column
==========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Separate data in table according to a column.

Missing values (na) are ignored.

Reads data from stdin unless infile is given.

Usage
-----

Example::

   python data2bins.py --help

Type::

   python data2bins.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse
import math
import bisect
import CGAT.Experiment as E
import CGAT.CSV as CSV

class Outputter:
    def __init__(self, filename, headers = None):
        self.mFilename = filename
        self.mOutfile = open(filename,"w")
        self.mCounts = 0
        if headers:
            self.mOutfile.write( "\t".join(headers) + "\n" )

    def write(self, data ):
        self.mOutfile.write( "\t".join(map(str,data)) + "\n" )
        self.mCounts += 1

    def __del__(self):
        self.mOutfile.close()
        if self.mCounts == 0:
            os.remove( self.mFilename )

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: data2bins.py 2782 2009-09-10 11:40:29Z andreas $", 
                                    usage = globals()["__doc__"])

    parser.add_option("--column", dest="column", type="int",
                      help="column to split on."  )

    parser.add_option("--num-bins", dest="num_bins", type="int",
                      help="number of bins to create."  )

    parser.add_option("--method", dest="method", type="choice",
                      choices=("equal-sized-bins" ,),
                      help="method to use to bin data."  )

    parser.add_option("--no-headers", dest="has_headers", action="store_false",
                      help="matrix has no row/column headers."  )

    parser.add_option( "-p", "--output-filename-pattern", dest="output_filename_pattern", type="string" ,
                       help="OUTPUT filename with histogram information on aggregate coverages [%default].")


    parser.set_defaults(
        has_headers = True,
        method = "equal-sized-bins",
        column = 1,
        num_bins = 4,
        output_filename_pattern = "bin%i",
        )

    (options, args) = E.Start( parser )
    options.column -= 1

    if args:
        if args[0] == "-":
            infile = sys.stdin
        else:
            infile = open(args[0],"r")
    else:
        infile = sys.stdin

    fields, data = CSV.ReadTable( infile )

    c = options.column 
    values = [ float(x[c]) for x in data ]

    bins = []

    if options.method == "equal-sized-bins":
        increment = int(math.floor( float(len(values)) / options.num_bins ) )
        indices = range( 0, len(values) )
        indices.sort( key = lambda x: values[x] )
        for x in xrange( len(values)): values[indices[x]] = x
        bins = range( 0, len(values)-increment, increment )

    elif options.method == "pass":
        pass
    
    E.debug( "bins=%s" % str(bins))

    outputters = []
    for x in xrange(0,len(bins)):
        outputters.append( Outputter(options.output_filename_pattern % x, fields ) )
        
    ## output tables
    for x in xrange(0, len(data)):
        bin = bisect.bisect( bins, values[x]) - 1
        outputters[bin].write(data[x])
        
    ## stats
    if options.loglevel >= 1:
        options.stdlog.write("# bin\tstart\tcounts\tfilename\n" )
        for x in xrange(0,len(bins)):
            options.stdlog.write("# %i\t%f\t%i\t%s\n" % (x, bins[x], outputters[x].mCounts, outputters[x].mFilename) )
            
    E.info( "ninput=%i, noutput=%i" % (len(data), sum( (x.mCounts for x in outputters )) ))
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
WrapperExonerate.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, re, tempfile, subprocess, optparse

"""Wrapper for Exonerate
"""

import Experiment as E
import PredictionParser
import Genomics

class ExonerateError(Exception):
    pass
            
class Exonerate:

    mDefaultOptions = "-m p2g --showvulgar FALSE --showsugar FALSE --showcigar FALSE"
    mOptions = ""
    mOutputOptions = '--showalignment FALSE ' + \
                     ' --ryo "diy\t%S\t%ql\t%r\t%pi\t%ps\t%V\n" --showtargetgff FALSE --showquerygff FALSE'
    
    mExecutable = "exonerate"
    mStdout = sys.stdout
    mStderr = sys.stderr
    
    def __init__( self, options="", output_options=[]):
        
        self.mOptions = options
        self.mDoParse = True

        if output_options:
            o = []
            if "gff" in output_options:
                o.append( "--showtargetgff TRUE --showquerygff TRUE")
            else:
                o.append( "--showtargetgff FALSE --showquerygff FALSE")
                    
            if "alignment" in output_options:
                o.append( "--showalignment TRUE")
            else:
                o.append( "--showalignment FALSE")

            if "parsed" in output_options:
                o.append( '--ryo "diy\t%S\t%ql\t%r\t%pi\t%ps\t%V\n"')
                self.mDoParse = True
            else:
                self.mDoParse = False
                
            self.mOutputOptions = " ".join(o)
        
    def CreateTemporaryFiles( self ):
        """create temporary files."""
        self.mTempDirectory = tempfile.mkdtemp()
        self.mFilenameTempPeptide = self.mTempDirectory + "/peptide.fasta"
        self.mFilenameTempGenome = self.mTempDirectory + "/genome.fasta"        
        
    def DeleteTemporaryFiles( self ):
        """clean up."""
        os.remove( self.mFilenameTempPeptide )
        os.remove( self.mFilenameTempGenome )
        os.rmdir( self.mTempDirectory )

    def SetStderr( self, file = None):
        """set file for dumping stderr."""
        self.mStderr= file

    def SetStdout( self, file = None):
        """set file for dumping stderr."""
        self.mStdout= file
        
    def Run( self,
             peptide_sequence,
             genomic_sequence ):

        self.CreateTemporaryFiles()

        outfile = open(self.mFilenameTempPeptide, "w")
        outfile.write(">%s\n%s\n" % ("peptide", peptide_sequence))
        outfile.close()

        outfile = open(self.mFilenameTempGenome, "w")
        outfile.write(">%s\n%s\n" % ("genome", genomic_sequence))
        outfile.close()
        
        statement = string.join( map(str, (
            self.mExecutable,
            self.mDefaultOptions,
            self.mOptions,
            self.mOutputOptions,
            "--query",
            self.mFilenameTempPeptide,
            "--target",
            self.mFilenameTempGenome,
            )), " " )

        if self.mLogLevel >= 3:
            print "# statement: %s" % statement

        s = subprocess.Popen( statement,
                              shell = True,
                              stdout = subprocess.PIPE,
                              stderr = subprocess.PIPE,
                              cwd = self.mTempDirectory,
                              close_fds = True)                              

        (out, err) = s.communicate()

        if s.returncode != 0:
            raise ExonerateError, "Error in calculating Exonerate\n%s" % err
            
        self.mStdout.write( out )
        self.mStderr.write( err )

        self.DeleteTemporaryFiles()

        parser = PredictionParser.PredictionParserExonerate()
        if self.mDoParse:
            return parser.Parse( out, peptide_sequence, genomic_sequence )
        

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: WrapperExonerate.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-p", "--peptide", dest="input_filename_peptide", type="string",
                      help="input filename with peptide sequence.",
                      metavar="FILE" )

    parser.add_option("-g", "--genome", dest="input_filename_genome", type="string",
                      help="input filename with genome sequence.",
                      metavar="FILE" )

    parser.add_option("-R", "--range-genome", dest="range_genome", type="string",
                      help="range on genome sequence.")

    parser.add_option("-P", "--range-peptide", dest="range_peptide", type="string",
                      help="range on peptide sequence.")

    parser.add_option("-d", "--output", dest="output_options", action="append",
                      help="output options [gff|alignment]" )

    parser.add_option("-o", "--options", dest="options", type="string",
                      help="options to exonerate" )

    parser.set_defaults( \
        input_filename_peptide = None,
        input_filename_genome = None,
        output_filename_stdout = "-",
        output_filename_stderr = "-",        
        options = "",
        range_genome = None,
        range_peptide = None,
        id_peptides = None,
        id_genomes = None,
        output_options = []
        )

    (options, args) = E.Start( parser ) 

    if options.range_genome: options.range_genome = map(int, options.range_genome.split(","))
    if options.range_peptide: options.range_peptide = map(int, options.range_peptide.split(","))    

    wrapper = Exonerate( options=options.options, output_options=options.output_options )
    wrapper.mLogLevel = options.loglevel

    if options.loglevel >= 2:
        print "# reading peptide sequence."
    peptide_sequences = Genomics.ReadPeptideSequences( open(options.input_filename_peptide, "r") )
    
    if options.loglevel >= 2:
        print "# reading genome sequence."
    genome_sequences = Genomics.ReadGenomicSequences( open(options.input_filename_genome, "r"), do_reverse = 0 )    

    if not options.id_peptides:
        options.id_peptides= peptide_sequences.keys()
    if not options.id_genomes:
        options.id_genomes= genome_sequences.keys()
        
    for x in options.id_peptides:
        ps = peptide_sequences[x]
        if options.range_peptide:
            ps = ps[options.range_peptide[0]:options.range_peptide[1]]
        for y in options.id_genomes:
            gs = genome_sequences[y]
            if options.range_genome:
                gs = gs[options.range_genome[0]:options.range_genome[1]]

            if options.loglevel >= 2:
                print "# aligning peptide of length %i: %s to genome of length %i: %s" % (len(ps), x, len(gs), y)
            result = wrapper.Run( ps, gs )

            print result
            
    E.Stop()
        




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gtf2reads.py - sample reads from genes
======================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script converts a :term:`gtf` formatted file into a :term:`fasta` formatted file of 
reads. The reads are sampled from the gene structures defined in the file.

Reads are sampled from a normal distribution with (read_length_mean,
read_length_stddev).

There are two modeling schemas:
   1 uniform expression: the sampling aims at a given
     statistical coverage. Each position in a transcript
     is equally likely to be a starting point of a sequencing
     read. The sampling procedure::

         1. estimate lvsequence = lsequence * gauss( coverage_mean, coverage_stddev)
         Sample until sum of residues in reads > lvsequence

            1. positions of read: uniformly from (1/lsequence)
            2. length of read: drawn from N(<read length>,stddev(read_length))
            3. strand of read: uniformly from (1/2)
            4. reject read if less than minimum size (min_read_length)

   Note that the effective read length and coverage will lie below the input
   values, as read lengths are constricted by the transcript length.

   2 sample according to expression profile
      TO BE IMPLEMENTED

For cross-species comparison, a mutation rate can be applied
from a normal distribution with (mutation_mean, mutation_stddev).


Usage
-----

Example::

   python gtf2reads.py --help

Type::

   python gtf2reads.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import types
import random
import CGAT.GTF as GTF
import CGAT.Experiment as E
import CGAT.IndexedFasta as IndexedFasta
import CGAT.Genomics as Genomics
import CGAT.SequencePairProperties as SequencePairProperties
import CGAT.Iterators as Iterators

import numpy
import numpy.random
import random

def getMutatedSequence( sequence, divergence ):
    """sample number of events from a Poisson distribution.

    This is a very hacky, simple mutator that does not take into account
    multiple substitutions per site and/or a substitution matrix.
    Only use for small divergences.
    """

    lsequence = len(sequence)
    nmutate = numpy.random.poisson( float(lsequence) * divergence )
    sequence = list(sequence.upper())

    for pos in random.sample(xrange(lsequence), nmutate):
        c = sequence[pos]
        x = c
        while x == c: x = random.choice( "ACGT" )
        sequence[pos] = x

    return "".join(sequence)

##------------------------------------------------------------
def main():

    parser = E.OptionParser( version = "%prog version: $Id: gtf2reads.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome [default=%default]."  )

    parser.add_option( "-p", "--output-filename-pattern", dest="output_filename_pattern", type="string" ,
                       help="OUTPUT filename with histogram information on aggregate coverages [%default].")

    parser.add_option( "--read-length-mean", dest="read_length_mean", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--read-length-std", dest="read_length_stddev", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--coverage-mean", dest="coverage_mean", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--coverage-std", dest="coverage_stddev", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--ds-mean", dest="ds_mean", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--ds-std", dest="ds_stddev", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--error-mean", dest="error_mean", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--error-std", dest="error_stddev", type="float",
                      help="simulation parameter [default=%default]."  )

    parser.add_option( "--min-read-length", dest="min_read_length", type="int",
                      help="minimum read length [default=%default]."  )

    parser.add_option( "--sample-size", dest="sample_size", type="int",
                      help="randomly sample from selected transcripts [default=%default]."  )

    parser.add_option( "--test", dest="test", type="int",
                      help="test with # first entries [default=%default]."  )

    parser.add_option( "--mode", dest="mode", type="choice",
                       choices=("genes", "transcripts" ),
                       help="use genes or transcripts [default=%default]."  )



    parser.set_defaults(
        genome_file = None,
        read_length_mean = 200.0,
        read_length_stddev = 20.0,
        coverage_mean = 2.0,
        coverage_stddev = 1.0,
        ds_mean = None,
        ds_stddev = None,
        error_mean = None,
        error_stddev = None,
        min_read_length = 50,
        test = None,
        mode = "transcripts",
        output_filename_pattern = None,
        output_format_id = "%010i",
        sample_size = 0,
        )

    (options, args) = E.Start( parser )
    
    assert options.genome_file, "please supply an indexed genome." 

    if options.output_filename_pattern:
        outfile_stats = open( options.output_filename_pattern % "stats", "w" )
        outfile_stats.write( "id\tlen\tnreads\tlen_mean\tlen_std\tcov_mean\tcov_std\n" )
        outfile_map = open( options.output_filename_pattern % "map", "w" )
        outfile_map.write( "id\ttranscript\n" )
    else:
        outfile_stats = None
        outfile_map = None

    genome = IndexedFasta.IndexedFasta( options.genome_file )

    ninput, noutput, nskipped = 0, 0, 0

    total_counts, total_read_lengths, total_len = [], [], 0
    total_pids = []
    total_error_pids = []

    if options.mode == "transcripts":
        iterator = GTF.transcript_iterator( GTF.iterator_filtered( GTF.iterator(options.stdin), feature="exon" ))
        getId = lambda x: x.transcript_id
    elif options.mode == "genes":
        iterator = GTF.flat_gene_iterator( GTF.iterator_filtered( GTF.iterator(options.stdin), feature="exon" ))
        getId = lambda x: x.gene_id

    if options.sample_size:
        iterator = Iterators.sample( iterator ) 
        
    if options.ds_mean:
        do_mutate = True
        pid_calc = SequencePairProperties.SequencePairPropertiesPID()
    else:
        do_mutate = False

    if options.error_mean:
        do_error = True
        pid_calc = SequencePairProperties.SequencePairPropertiesPID()
    else:
        do_error = False

    for gtfs in iterator:

        id = getId(gtfs[0])

        try:
            sequence = GTF.toSequence( gtfs, genome )
        except KeyError, msg:
            if options.loglevel >= 2:
                options.stdlog.write( "# skipping %s: %s\n" % (id, msg))
            nskipped += 1
            continue

        lsequence = len(sequence)
        
        if lsequence <= options.min_read_length * 2:
            if options.loglevel >= 2:
                options.stdlog.write( "# skipping %s - sequence is too short: %i\n" % (id, lsequence) )
            nskipped += 1
            continue

        ninput += 1

        if do_mutate:
            new_sequence = getMutatedSequence( sequence, options.ds_mean )
            pid_calc.loadPair( sequence, new_sequence)
            pid = pid_calc.mPID 
            total_pids.append( pid )
            sequence = new_sequence
        else:
            pid = 100.0

        if options.loglevel >= 2:
            options.stdlog.write("# processing %s - len=%i\n" % (id, lsequence) )
            options.stdlog.flush()

        total_len += lsequence
        lvsequence = lsequence * random.gauss( options.coverage_mean, options.coverage_stddev )
        
        covered = 0
        counts = numpy.zeros( lsequence )
        nreads = 0

        error_pids, read_lengths = [], []
        
        while covered < lvsequence:
            
            read_length = int(random.gauss( options.read_length_mean, options.read_length_stddev ))
            positive = random.randint(0,1)

            if positive:
                start = random.randint(0,lsequence)
                end = min( lsequence, start + read_length )
            else:
                end = random.randint(0,lsequence)
                start = max(0, end - read_length)
                
            read_length = end - start
            if read_length < options.min_read_length:
                continue
            
            segment = sequence[start:end]

            if not positive:
                segment = Genomics.complement( segment )

            noutput += 1

            if do_error:
                new_segment = getMutatedSequence( segment, options.error_mean )
                pid_calc.loadPair( segment, new_segment)
                pid = pid_calc.mPID 
                error_pids.append( pid )
                segment = new_segment
            else:
                pid = 100.0
                
            options.stdout.write( ">%s\n%s\n" % (options.output_format_id % noutput, segment ) )
            
            if outfile_map:
                outfile_map.write( "%s\t%s\n" % (id, options.output_format_id % noutput) )

            for x in range(start,end):
                counts[x] += 1

            nreads += 1

            covered += read_length
            read_lengths.append(read_length)
            
        if options.loglevel >= 2:
            options.stdout.write( "# transcript %s: len=%i, nreads=%i, len_mean=%.2f, len_std=%.2f, cov_mean=%.2f, cov_stddev=%.2f\n" % (id, 
                                                                                                                                         lsequence,
                                                                                                                                         nreads, 
                                                                                                                                         numpy.mean( read_lengths ),
                                                                                                                                         numpy.std( read_lengths ),
                                                                                                                                         numpy.mean( counts ),
                                                                                                                                         numpy.std( counts) ))
        if outfile_stats:
            outfile_stats.write( "%s\t%i\t%i\t%.2f\t%.2f\t%.2f\t%.2f\n" % (id, 
                                                                           lsequence,
                                                                           nreads, 
                                                                           numpy.mean( read_lengths ),
                                                                           numpy.std( read_lengths ),
                                                                           numpy.mean( counts ),
                                                                           numpy.std( counts) ))


        total_counts += list(counts)
        total_read_lengths += read_lengths
        total_error_pids += error_pids

        if options.test and ninput >= options.test:
            break

        if options.sample_size and ninput >= options.sample_size:
            break

    if options.loglevel >= 1:
        output = ["len=%i, nreads=%i" % ( total_len,
                                          noutput ) ]
        output.append( "len_mean=%.2f, len_std=%.2f, cov_mean=%.2f, cov_stddev=%.2f" % ( 
                numpy.mean( total_read_lengths ),
                numpy.std( total_read_lengths ),
                numpy.mean( total_counts ),
                numpy.std( total_counts) ))

        no_uncovered = [ x for x in total_counts if x > 0 ]

        output.append( "cov0_mean=%.2f, cov0_stddev=%.2f" % (numpy.mean( no_uncovered ),
                                                             numpy.std( no_uncovered ) ) )

        if do_mutate:
            output.append( "pid_mean=%.2f, pid_std=%.2f" % (numpy.mean( total_pids ), numpy.std( total_pids ) ))

        if do_error:
            output.append( "pid_error_mean=%.2f, pid_error_std=%.2f" % (numpy.mean( total_error_pids ), numpy.std( total_error_pids ) ))
            
            
        options.stdlog.write( "# effective: %s\n" % ", ".join( output ))

    if options.loglevel >= 1:
        options.stdlog.write( "# ninput=%i, noutput=%i, nskipped=%i\n" % (ninput, noutput, nskipped ) )

    E.Stop()

if __name__ == '__main__':
    main()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
BlastAlignments.py - tools for working with alignments
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import re, string, sys

try: import alignlib
except ImportError: pass

class Map:
    '''a blast alignment.'''

    def __init__(self ):
        (self.mQueryToken, self.mSbjctToken, self.mEvalue,
         self.mQueryFrom, self.mQueryTo, self.mQueryAli,
         self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli ) =\
         ("", "", 0, 0, 0, "", 0, 0, "" )
        self.mIsExpanded = False
        self.mMapQuery2Sbjct = None

    def Read( self, line ):
        
        (self.mQueryToken, self.mSbjctToken, self.mEvalue,
         self.mQueryFrom, self.mQueryTo, self.mQueryAli,
         self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli) = line[:-1].split("\t")[:9]

        (self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo) = map(
            int, (self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo))

        self.mEvalue = float(self.mEvalue)

    def __str__( self ):

        return string.join( map(str, (
            self.mQueryToken, self.mSbjctToken, self.mEvalue,
            self.mQueryFrom, self.mQueryTo, self.mQueryAli,
            self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli)), "\t")

    def MapRange( self, query_token, query_from, query_to ):
        """map something."""
        map_query2sbjct = alignlib.makeAlignataVector()
        alignlib.fillAlignataCompressed( map_query2sbjct,
                                         self.mQueryFrom, self.mQueryAli,
                                         self.mSbjctFrom, self.mSbjctAli)
        
        new_from = 0

        if query_from <= map_query2sbjct.getRowTo():
            x = max(query_from, self.mQueryFrom)
            while map_query2sbjct.mapRowToCol(x) == 0:
                x += 1
                if x > map_query2sbjct.getRowTo(): break
            else:
               new_from = map_query2sbjct.mapRowToCol(x)

        new_to = 0

        if query_to >= map_query2sbjct.getRowFrom():
            x = min(query_to, self.mQueryTo)
            while map_query2sbjct.mapRowToCol(x) == 0:
                x -= 1
                if x < map_query2sbjct.getRowFrom(): break
            else:
               new_to = map_query2sbjct.mapRowToCol(x)
        
        return self.mSbjctToken, new_from, new_to


    def Expand( self ):
        if not self.mIsExpanded:
            self.mMapQuery2Sbjct = alignlib.makeAlignataVector()
            alignlib.fillAlignataCompressed( self.mMapQuery2Sbjct,
                                             self.mQueryFrom, self.mQueryAli,
                                             self.mSbjctFrom, self.mSbjctAli )
        self.mIsExpanded = True
        
    def Clear( self ):
        if self.mIsExpanded:
            self.mMapQuery2Sbjct = None
            self.mIsExpanded = False
        
    def Contract( self ):
        if self.mIsExpanded:
            self.mQueryAli, self.mSbjctAli = alignlib.writeAlignataCompressed( self.mMapQuery2Sbjct )
            self.mQueryFrom = self.mMapQuery2Sbjct.getRowFrom()
            self.mQueryTo = self.mMapQuery2Sbjct.getRowTo()        
            self.mSbjctFrom = self.mMapQuery2Sbjct.getColFrom()
            self.mSbjctTo = self.mMapQuery2Sbjct.getColTo()
            self.mIsExpanded = False

    def GetClone( self ):
        """get copy of self.
        """

        m = Map()
        
        (m.mQueryToken, m.mSbjctToken, m.mEvalue,
         m.mQueryFrom, m.mQueryTo, m.mQueryAli,
         m.mSbjctFrom, m.mSbjctTo, m.mSbjctAli,
         m.mIsExpanded) = \
         (self.mQueryToken, self.mSbjctToken, self.mEvalue,
          self.mQueryFrom, self.mQueryTo, self.mQueryAli,
          self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
          self.mIsExpanded)

        if self.mIsExpanded:
            m.mMapQuery2Sbjct = alignlib.makeAlignataVector()
            alignlib.copyAlignata( m.mMapQuery2Sbjct, self.mMapQuery2Sbjct )
            
        return m
        
    def MapAlignment( self, map_query = None, map_sbjct = None ):

        self.Expand()
        
        tmp = alignlib.makeAlignataVector()

        if map_query:
            tmp.Clear()
            map_query.Expand()
            alignlib.combineAlignata( tmp,
                                      map_query.mMapQuery2Sbjct,
                                      self.mMapQuery2Sbjct,
                                      alignlib.RR )
            map_query.Clear()
            alignlib.copyAlignata( self.mMapQuery2Sbjct, tmp )
            self.mQueryToken = map_query.mSbjctToken
            
        if map_sbjct:
            tmp.Clear()
            map_sbjct.Expand()
            alignlib.combineAlignata( tmp,
                                      self.mMapQuery2Sbjct,
                                      map_sbjct.mMapQuery2Sbjct,
                                      alignlib.CR )
            map_sbjct.Clear()
            alignlib.copyAlignata( self.mMapQuery2Sbjct, tmp )
            self.mSbjctToken = map_sbjct.mSbjctToken
            
        self.Contract()
        
        return self.IsOk()

    def Reverse( self ):
        (self.mQueryToken, self.mSbjctToken, 
         self.mQueryFrom, self.mQueryTo, self.mQueryAli,
         self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli ) = \
        ( self.mSbjctToken, self.mQueryToken,
          self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli, 
          self.mQueryFrom, self.mQueryTo, self.mQueryAli )
        
    def IsOk( self ):
        return self.mMapQuery2Sbjct.getRowFrom() != 0 and \
               self.mMapQuery2Sbjct.getRowTo() != 0 and \
               self.mMapQuery2Sbjct.getColFrom() != 0 and \
               self.mMapQuery2Sbjct.getColTo() != 0

        
class SelfMap( Map ):
    def __init__(self):
        Map.__init__(self)
    def __str__(self):
        return ""
    def Read(line):
        pass
    def MapRange( self, query_token, query_from, query_to):
        return query_token, query_from, query_to
        
def ReadMap( file, multiple = False ):
    """read a map from a file.

    If multiple is true, return a list of mappings.
    """
    m = {}
    for line in file:
        if line[0] == "#": continue
        mm = Map()
        mm.Read(line)
        if multiple:
            if mm.mQueryToken not in m: m[mm.mQueryToken] = []
            m[mm.mQueryToken].append(mm)
        else:
            m[mm.mQueryToken] = mm
    return m
        

class Link( Map ):

    mFormat = 0
    
    def __init__(self):
        pass

    def Read( self, line ):

        data = line[:-1].split("\t")

        try:
            (self.mQueryToken, self.mSbjctToken, self.mEvalue,
             self.mQueryFrom, self.mQueryTo, self.mQueryAli,
             self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
             self.score, self.mPercentIdentity, 
             self.mQueryLength, self.mSbjctLength) = data[:13]

            (self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo,
             self.score, self.mPercentIdentity, self.mQueryLength, self.mSbjctLength) = map(
                int, (self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo,
                      self.score, self.mPercentIdentity, self.mQueryLength, self.mSbjctLength))
            self.mQueryLength = max( self.mQueryTo, self.mQueryLength )
            self.mSbjctLength = max( self.mSbjctTo, self.mSbjctLength )         
            self.mEvalue = float(self.mEvalue)
        except ValueError:
            raise ValueError, line

        self.mBitScore = 0.0
        self.mFormat = 1
        
        if (len(data) == 14):
            self.mFormat = 2
            self.mBitScore = float(data[13])

    def Reverse( self ):
        """reverse alignment."""
        Map.Reverse( self )
        self.mQueryLength, self.mSbjctLength = self.mSbjctLength, self.mQueryLength
        
    def __str__( self ):

        if self.mFormat == 1:
            return string.join( map(str, (
                self.mQueryToken, self.mSbjctToken, self.mEvalue,
                self.mQueryFrom, self.mQueryTo, self.mQueryAli,
                self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
                self.score, self.mPercentIdentity, 
                self.mQueryLength, self.mSbjctLength)), "\t")
        elif self.mFormat == 2:
            return string.join( map(str, (
                self.mQueryToken, self.mSbjctToken, self.mEvalue,
                self.mQueryFrom, self.mQueryTo, self.mQueryAli,
                self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli,
                self.score, self.mPercentIdentity, 
                self.mQueryLength, self.mSbjctLength, self.mBitScore)), "\t")
            
def iterator_links( infile ):
    """a simple iterator over all entries in a file."""
    while 1:
        line = infile.readline()
        if not line: return
        if line[0] == "#": continue
        v = Link()
        v.Read( line )
        yield v

    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
plot_histogram.py - create line plot of tabular data
====================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Reads data in tabular format with observations in rows in
different samples in columns. The first column provides the
bins.

Missing values (na) are ignored.

Reads data from stdin unless infile is given.

Usage
-----

Example::

   python plot_histogram.py --help

Type::

   python plot_histogram.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import re
import string
import os
import getopt
import time
import optparse
import array
import CGAT.Experiment as E
import CGAT.Histogram as Histogram
import CGAT.MatlabTools as MatlabTools
import numpy
import CGAT.IOTools as IOTools
import numpy.ma

def main():

    parser = E.OptionParser( version = "%prog version: $Id: plot_histogram.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-l", "--legend", dest="legend", type="string",
                      help="legend for plot [default=%default]."  )
    parser.add_option("-t", "--title", dest="title", type="string",
                      help="title for plot [default=%default]."  )
    parser.add_option("-p", "--hardcopy", dest="hardcopy", type="string",
                      help="filename for hardcopy of plot. The extension defines the format. Known extensions are: 'emf, eps, jpeg, jpg, pdf, png, ps, raw, rgba, svg, svgz' [default=%default].", metavar = "FILE"  )
    parser.add_option("", "--xrange", dest="xrange", type="string",
                      help="x viewing range of plot [default=%default]."  )
    parser.add_option("", "--yrange", dest="yrange", type="string",
                      help="y viewing range of plot[default=%default]."  )
    parser.add_option("-o", "--logscale", dest="logscale", type="string",
                      help="use logscale on x, y or xy [default=%default]"  )
    parser.add_option("-x", "--xtitle", dest="xtitle", type="string",
                      help="title for x axis [default=%default]"  )
    parser.add_option("-y", "--ytitle", dest="ytitle", type="string",
                      help="title for y axis [default=%default]"  )
    parser.add_option("-d", "--dpi", dest="dpi", type="int",
                      help="dpi of images [default=%default]"  )
    parser.add_option("-n", "--normalize", dest="normalize", action = "store_true",
                      help="normalize histograms [default=%default]"  )
    parser.add_option( "--cumulate", dest="cumulate", action = "store_true",
                      help="calculate cumulative histogram [default=%default]."  )
    parser.add_option( "--reverse-cumulate", dest="reverse_cumulate", action = "store_true",
                      help="calculate cumulative histogram in reverse order [default=%default]."  )
    parser.add_option( "--legend-location", dest="legend_location", type="choice",
                       choices=("upper left", "upper right", "lower left", "lower right", "center", "center right", "center left", "none"),
                       help="location of legend [default=%default]"  )
    parser.add_option( "--backend", dest="backend", type="string",
                      help="backend to use [Agg|SVG|PS] [default=%default]"  )
    parser.add_option( "--symbols", dest="symbols", type="string",
                      help="symbols to use for each histogram [steps|...] [default=%default]."  )
    parser.add_option( "--dump", dest="dump", action="store_true",
                      help="dump data for debug purposes [default=%default]."  )
    parser.add_option("-c", "--columns", dest="columns", type="string",
                      help="columns to use for plotting [default=%default].")
    parser.add_option("--truncate", dest="truncate", action="store_true",
                      help="truncate date within x range. If not set, xrange is simply a viewing range [default=%default].")
    parser.add_option("--as-lines", dest="as_lines", action="store_true",
                      help="plot only lines, no symbols [default=%default].")
    parser.add_option("--noheaders", dest="headers", action="store_false",
                      help="do not take first input line as header [default=%default].")
    parser.add_option("--stacked", dest="stacked", action="store_true",
                      help="do a stacked plot [default=%default].")
    parser.add_option("--add-function", dest="function", type="string",
                      help="add a function to the plot [default=%default]." )
    parser.add_option("--add-error-bars", dest="error_bars", type="choice",
                      choices=("interleaved", "blocked"),
                      help="add error bars. The input format is 'interleaved' or 'blocked'. In the interleaved format the error follows each column. I the blocked format first the data, then the errors in the same order [default=%default]." )

    parser.set_defaults(
        legend = None,
        title = None,
        hardcopy = None,
        logscale = None,
        xtitle = None,
        ytitle = None,
        xrange = None,
        yrange = None,        
        normalize = None,
        columns = "all",
        headers = True,
        legend_location = "upper right",
        backend = "cairo",
        symbols = "g-D,b-h,r-+,c-+,m-+,y-+,k-o,g-^,b-<,r->,c-D,m-h",
        dump = False,
        truncate = False,
        cumulate = False,
        reverse_cumulate = False,
        function = None,
        add_error_bars = None,
        as_lines = False,
        stacked = False,
        dpi = 80,
        )

    (options, args) = E.Start( parser )

    # import matplotlib/pylab. Has to be done here
    # for batch scripts without GUI.
    import matplotlib
    if options.hardcopy: matplotlib.use( "cairo" )
    import pylab

    # put this method here (because it requires pylab)
    def doStackedPlot( data, legend ):

        colors = ["red",
                  "blue",
                  "green",
                  "cyan",
                  "magenta",
                  "yellow",
                  "brown",
                  "silver",
                  "purple",
                  "lightyellow",
                  "black",
                  "ivory",
                  "pink",
                  "orange",
                  "gray",
                  "teal"]

        ax = data[:,0]
        xvals = numpy.concatenate((ax,ax[::-1]))
        y_top = numpy.zeros(len(ax))

        min_y = min(data[:,1:].flat)
        max_y = min_y
        new_legend, dummy_lines = [], []

        for i in range(1,len(legend)):
            new_y_top = y_top + data[:,i]
            yvals = numpy.concatenate((new_y_top,y_top[::-1]))
            p = pylab.fill(xvals,
                           yvals,
                           colors[i % len(colors)])

            y_top = new_y_top
            max_y = max(y_top)

            dummy_lines.append(pylab.plot( xvals,
                                           yvals,
                                           colors[i % len(colors)] ) )

            new_legend.append( legend[i] )

        if not options.xrange:
            options.xrange=min(data[:,0]),max(data[:,0])

        if not options.yrange:
            options.yrange=0,max_y

        return dummy_lines, new_legend


    if options.as_lines:
        options.symbols = []
        for y in ("-",":","--"):
            for x in "gbrcmyk":
                options.symbols.append( y+x )
    else:
        options.symbols = options.symbols.split(",")
    
    if options.xrange: options.xrange = map(float, options.xrange.split(","))
    if options.yrange: options.yrange = map(float, options.yrange.split(","))    

    # Added support for (inclusive) range format: "1,3,5,7-100"  (Gerton 13/12/06)
    if options.columns != "all":
        cols = []
        for d in options.columns.split(','):
            colopts = d.split('-')
            if len(colopts)==2: cols += range( int(colopts[0]),int(colopts[1])+1 )
            else: cols += [int(d)-1]
        options.columns = cols

    if args:
        if args[0] == "-":
            infile = sys.stdin
        else:
            infile = open(args[0],"r")
    else:
        infile = sys.stdin
        
    if options.truncate:
        xr = options.xrange
    else:
        xr = None

    data, legend = IOTools.readTable( infile,
                                      numeric_type=numpy.float,
                                      take=options.columns,
                                      headers = options.headers,
                                      truncate= xr )
    
    if infile != sys.stdin: infile.close()
    if len(data) == 0: # or data == None:
        E.info("empty table: no plot" )
        E.Stop()
        return

    nrows, ncols = data.shape

    ## note: because of MA, iteration makes copy of slices
    ## Solution: inplace edits.
    if options.cumulate:
        if options.add_error_bars:
            raise "can not add error bars to cumulative histogram."
        if data.mask.any():
            # cumsum does not work with masked arrays, so do it manually
            for y in range(1, ncols):
                c = 0
                for x in range(0, nrows):
                    if not data.mask[x,y]:
                        data[x,y] += c
                        c = data[x,y]
        else:
            for x in range(1,ncols):
                data[:,x] = data[:,x].cumsum()

    elif options.reverse_cumulate:
        if options.add_error_bars:
            raise "can not add error bars to cumulative histogram."
        if data.mask.any():
            l = [0] * ncols
            for x in range(nrows-1,-1,-1):
                for y in range(1, ncols):
                    if not data.mask[x,y]:
                        data[x,y] += l[y]
                        l[y] = data[x,y]
        else:
            l = [0] * ncols
            for x in range(nrows-1,-1,-1):
                for y in range(1, ncols):
                    data[x,y] += l[y]
                    l[y] = data[x,y]

    if options.normalize:
        if options.add_error_bars:
            raise "can not add error bars to normalized histogram."
        if data.mask.any():
            m = [0] * ncols
            for x in range(nrows):
                for y in range(1, ncols):
                    if not data.mask[x,y]:
                        m[y] = max( m[y], float(data[x,y]) )

            for y in range(1, ncols):
                if m[y] == 0: m[y] = 1.0

            for x in range(nrows):
                for y in range(1, ncols):
                    data[x,y] = data[x,y] / m[y]
        else:
            for x in range(1,ncols):
                m = float(data[:,x].max())
                data[:,x] /= m

    if options.legend:
        legend = options.legend.split(",")
        
    if options.dump:
        for d in data:
            print d

    if options.title:
        pylab.title( options.title )

    if options.xtitle:
        pylab.xlabel( options.xtitle )
    else:
        pylab.xlabel( legend[0] )

    if options.ytitle:
        pylab.ylabel( options.ytitle )
        
    lines = []
    # use dummy_lines to workaround a bug in errorbars that
    # causes the line styles to be set incorrectly.
    dummy_lines = []
    new_legend = []

    if options.error_bars:
        if options.error_bars == "interleaved":
            step_size = 2
            max_size = len(legend)
        elif options.error_bars == "blocked":
            step_size = 1
            max_size = (len(legend) -1 ) / 2
    else:
        step_size = 1
        max_size= len(legend)

    if options.stacked:
        dummy_lines, new_legend = doStackedPlot( data, legend )
    else:
        nplotted = 0
        nskipped = 0
        for x in range(1, max_size, step_size):

            s = options.symbols[nplotted % len(options.symbols)]

            yvals = data[:,x]

            xvals = numpy.ma.masked_array(data[:,0], numpy.ma.getmask(yvals))

            xvals = xvals.compressed()
            yvals = yvals.compressed()            

            if len(xvals) == 0:
                E.warn( "skipped empty column %i: %s" % (x,legend[x] ) )

            if options.error_bars == "interleaved":
                yerr = data[:,x+1]
                yerr = yerr.compressed()
            else:
                yerr = None

            lines.append(pylab.errorbar( xvals,
                                         yvals,
                                         yerr = yerr,
                                         fmt = s ) )

            dummy_lines.append(pylab.plot( xvals,
                                           yvals,
                                           s ) )

            new_legend.append( legend[x] )

            nplotted += 1

        E.info("nplotted=%i, nskipped=%i" % (nplotted, nskipped))

    if len(lines) == 0:
        E.Stop()
        return 

    if options.legend_location != "none":
        pylab.figlegend( dummy_lines,
                         new_legend,
                         options.legend_location )

    if options.logscale:
        if "x" in options.logscale:
            pylab.gca().set_xscale('log')
        if "y" in options.logscale:
            pylab.gca().set_yscale('log')

    if options.xrange:
        pylab.xlim( options.xrange )
        
    if options.yrange:
        pylab.ylim( options.yrange )

    if options.function:
        xstart, xend = pylab.gca().get_xlim()
        increment = (xend - xstart) / 100.0
        exec("f = lambda x: %s" % options.function ) in locals()
        xvals, yvals = [], []
        for x in range(0,100):
            xvals.append( xstart )
            yvals.append( f(xstart) )
            xstart += increment
        xvals.append( xstart )
        yvals.append( f(xstart) )
        
        pylab.plot( xvals, yvals )

    if options.hardcopy:
        pylab.savefig( os.path.expanduser(options.hardcopy), dpi = options.dpi )
    else:
        pylab.show()

    E.Stop()

if __name__ == "__main__":
    sys.exit(main())



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
convert_time2seconds.py -
=============================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert output of the time command into seconds:

input: for example something like

1h33m47.32s

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import string
import re
import optparse

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    for line in sys.stdin:

        if line[0] == "#": continue

        data = string.split(line[:-1], "\t")

        new_fields = []

        for field in data:

            x = re.match("^(\d+)h(\d+)m([\d.]+)s", field)
            if not x:
                x = re.match("^(\d+)m([\d.]+)s", field)
                if x:
                    minutes, seconds = x.groups()
                else:
                    new_fields.append(field)
                    continue
                hours = "0"
            else:
                hours, minutes, seconds = x.groups()


            new_fields.append( "%.2f" % (string.atof(hours) * 3600.0 + string.atof(minutes) * 60.0 + string.atof(seconds)))

        print string.join(new_fields, "\t")

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



    



    




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
correlate_fasta_identifier.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python correlate_fasta_identifier.py --help

Type::

   python correlate_fasta_identifier.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile

USAGE="""python %s [OPTIONS] filename < stdin > stdout

Version: $Id: correlate_fasta_identifier.py 14 2005-08-09 15:24:07Z andreas $

Given two :term:`fasta` formatted files, substitute
identifiers in stream with those given in file with filename.

Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
"""

param_loglevel = 0

param_long_options=["verbose=", "help", "version"]
param_short_options="v:h"

##------------------------------------------------------------
if __name__ == '__main__':

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print USAGE, msg
        sys.exit(2)
        
    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print USAGE
            sys.exit(0)

    if len(args) != 1:
        print "please supply filename with replacement identifiers."
        print USAGE
        sys.exit(1)


    identifiers = []
    infile = open(args[0], "r")
    for line in infile:
        if line[0] == ">":
            identifiers.append( line[1:string.find(" ", line)] )
    infile.close()

    x = 0
    for line in sys.stdin:
        if line[0] == ">":
            if x >= len(identifiers): raise "different number of sequences."            
            line = ">" + identifiers[x] + "\n"
            x += 1
        print line[:-1]
    
            



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat.py - frontend for cgat scripts
===================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Usage
-----

To get help for a specific command, type::

    cgat <command> --help

'''

import os
import sys
import glob
import imp

def main():

    argv = sys.argv

    path = os.path.abspath( os.path.dirname(__file__) )

    if argv[1] == "--help" or argv[1] == "-h":
        print(globals()["__doc__"])
        print("The list of available commands is:\n" )
        print( "\n".join( sorted([os.path.basename(x)[:-3] for x in glob.glob( os.path.join( path, "*.py") )]) ) )
        return

    command = argv[1]

    (file, pathname, description ) = imp.find_module( command, [path,] )
    module = imp.load_module( command, file, pathname, description)
    module.main( sys.argv )
    
if __name__ == "__main__":
    sys.exit( main() )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
tree_species2genes.py - collapse leaves with the same species.
==============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

-p, --pattern-species=          regex pattern to extract species from identifier
-g, --genes=                    filename with list of genes per species

Usage
-----

Example::

   python tree_species2genes.py --help

Type::

   python tree_species2genes.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import popen2

from Bio.Nexus import Nexus
from Bio.Nexus.Nodes import Node

import CGAT.Experiment as E
import CGAT.BlastAlignments as BlastAlignments
import CGAT.Genomics as Genomics

param_loglevel = 1

param_long_options=["verbose=", "help",
                    "pattern-species=",
                    "genes=", "version"]

param_short_options="v:hp:g:"

param_pattern_species = "^([^@:]+)[@:]"

param_filename_genes = None

if __name__ == "__main__":

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print globals()["__doc__"], msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print globals()["__doc__"]
            sys.exit(0)
        elif o in ("-p", "--pattern-species"):
            param_pattern_species = a
        elif o in ("-g", "--genes"):
            param_filename_genes = a

    if not param_filename_genes:
        print "please specify a filename with genes"
        print globals()["__doc__"]
        sys.exit(1)
    
    print E.GetHeader()
    print E.GetParams()

    rx = re.compile( param_pattern_species )
    infile = open(param_filename_genes, "r")
    map_species2genes = {}
    for line in infile:
        if line[0] == "#": continue
        gene = line[:-1].split("\t")[0]
        species = rx.search( gene ).groups()[0]
        if species not in map_species2genes:
            map_species2genes[species] = []
        map_species2genes[species].append( gene )
    
    nexus = TreeTools.Newick2Nexus( sys.stdin )

    TreeTools.Species2Genes( nexus, map_species2genes )

    print TreeTools.Nexus2Newick( nexus )
    
    print E.GetFooter()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat_rebuild_extensions.py - rebuild all cython extensions
==========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script rebuilds all cython extensions in the source directory.

Some scripts in the repository make use of ``pyximport`` to compile
associated cython scripts with embedded C code. Theses scripts are
automatically re-compiled if the script has changed, but this process
can fail if:

   * the script is executed on a machine without a C-compiler
   * some underlying libraries have changed.

Thus, it is safer to rebuild all scripts on a machine with a C compiler
before running a script in production on a cluster, where not all nodes
might be fully configured for compilation.

Usage
-----

Example::

   python cgat_rebuild_extensions.py

Type::

   python cgat_rebuild_extensions.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse
import glob

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-i", "--test-option", dest="test_option", type="string",
                      help="test option [default=%default]."  )

    parser.set_defaults(
        test_option = "test"
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    files = glob.glob( os.path.join( os.path.dirname(__file__), "*.pyx" ))

    ## do sth
    ninput, nskipped, noutput = 0, 0, 0

    for f in files:
        E.info( "rebuilding %s" % f )
        ninput += 1
        prefix, suffix = os.path.splitext( f )
        for ext in (".c", ".pyxbldc" ):
            try: os.remove( prefix + ext )
            except OSError: pass
            
        dirname, basename = os.path.split( prefix )
        assert basename.startswith("_")
        
        scriptname = os.path.join( dirname, basename[1:] ) + ".py" 
        if not os.path.exists( scriptname ):
            E.warn("script %s does not exist - skipped" % scriptname)
            nskipped += 1
            continue
        
        E.info( "compiling %s" % scriptname )
        os.system( "%s %s --help > /dev/null" % (sys.executable, scriptname ) )

        noutput += 1

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )
    
    ## write footer and output benchmark information.
    E.Stop()
    
if __name__ == "__main__":
    sys.exit( main( sys.argv) )



"""
Orthologs.py - tools to deal with Leo's orthology pipeline.
===========================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

"""

import string, re

## patch to work around wrong gene assignments for BDGP
global_ignore_bdgp_genes = False

def FilterBDGP( data ):
    """remove genes that are

    transcripts from BDGP and genes from ENSEMBL.

    If only BDGP genes are present, keep them.
    
    """
    rx = re.compile( "CG.+-R." )
    n = []
    
    for d in data:
                    
        if d.mSchema == "dmel_vs_dmel":
            if (rx.match(d.mGene) and not rx.match(d.mTranscript)) or \
                   (not rx.match(d.mGene) and not rx.match(d.mTranscript)):
                continue
        n.append( d )
        
    return n

def GetGenes( transcripts ):
    """from a list of transcripts get genes.
    """
    genes = {}
    for t in transcripts:
        if t.mGene not in genes: genes[t.mGene] = []
        genes[t.mGene].append( t )
        
    return genes

class Transcript:

    mSeparator = "|"
    def __init__( self, a = None):
        if a:
            (self.mSchema, \
             self.mTranscript, \
             self.mGene, \
             self.mQuality) = a.split(self.mSeparator)

    def __str__(self):
        return self.mSeparator.join( (self.mSchema, self.mTranscript, self.mGene, self.mQuality) )
    
def ReadInterpretation( infile, separator,
                        genome1 = None, genome2 = None,
                        filter_restrict_genes1 = {},
                        filter_restrict_genes2 = {},
                        filter_remove_transcripts1 = {},
                        filter_remove_transcripts2 = {},
                        filter_restrict_transcripts1 = {},
                        filter_restrict_transcripts2 = {},
                        ):
    """read interpretation file.
    """
    pairs = []
    neliminated = 0
    for line in infile:
        if line[0] == "#": continue
        
        data = line[:-1].split("\t")
        weight = float(data[0])

        transcripts = []
        ## convert
        # Patch for invalid gene (0). These should not be here in the
        # first place.
        for x in range(1,len(data)):
            t = Transcript( data[x] )
            
            if t.mGene != "0":
                transcripts.append( t )

        if len(transcripts) < 2:
            continue

        first_prefix = transcripts[0].mSchema
        last_prefix  = transcripts[-1].mSchema
        for sep in range(1, len(transcripts)):
            if first_prefix != transcripts[sep].mSchema:
                transcripts1 = transcripts[:sep]
                transcripts2 = transcripts[sep:]
                break
        else:
            continue

        ## revert matches, so that the order is genome1, genome2
        if genome1 and genome2 and first_prefix == genome2 and last_prefix == genome1:
            transcripts2, transcripts1 = transcripts1, transcripts2
            first_prefix, last_prefix = last_prefix, first_prefix

        ## filter if genome don't fit.
        if first_prefix and last_prefix and genome1 and genome2 and\
               first_prefix != genome1 and last_prefix != genome2:
            continue

        if global_ignore_bdgp_genes:
            transcripts1 = FilterBDGP( transcripts1 )
            transcripts2 = FilterBDGP( transcripts2 )            

        ## apply various filters
        if filter_restrict_genes1: transcripts1 = filter( lambda x: x.mGene in filter_restrict_genes1, transcripts1)
        if filter_restrict_genes2: transcripts2 = filter( lambda x: x.mGene in filter_restrict_genes2, transcripts2)
        if filter_remove_transcripts1: transcripts1 = filter( lambda x: x.mTranscript not in filter_remove_transcripts1, transcripts1)
        if filter_remove_transcripts2: transcripts2 = filter( lambda x: x.mTranscript not in filter_remove_transcripts2, transcripts2)

        if filter_restrict_transcripts1: transcripts1 = filter( lambda x: x.mTranscript in filter_restrict_transcripts1, transcripts1)
        if filter_restrict_transcripts2: transcripts2 = filter( lambda x: x.mTranscript in filter_restrict_transcripts2, transcripts2)

        genes1 = GetGenes( transcripts1 )
        genes2 = GetGenes( transcripts2 )        

        if len(transcripts1) == 0 and len(transcripts2) == 0:
            neliminated += 1
            continue
        
        pairs.append( ( transcripts1, transcripts2, genes1, genes2, weight) )

    return pairs

def ReadOrphans( infile, separator,
                 genome1 = None, genome2 = None,
                 filter_restrict_genes1 = {},
                 filter_restrict_genes2 = {},
                 filter_remove_transcripts1 = {},
                 filter_remove_transcripts2 = {},
                 filter_restrict_transcripts1 = {},
                 filter_restrict_transcripts2 = {},
                 ):
    """read interpretation file.
    """
    pairs = []
    neliminated = 0
    for line in infile:
        if line[0] == "#": continue
        
        data = line[:-1].split("\t")

        transcripts = []
        ## convert
        # Patch for invalid gene (0). These should not be here in the
        # first place.
        for x in range(0,len(data)):
            t = Transcript( data[x] )
            
            if t.mGene != "0":
                transcripts.append( t )

        schema = transcripts[0].mSchema
        
        ## apply various filters        
        if schema == genome1:
            if filter_restrict_genes1: transcripts = filter( lambda x: x.mGene in filter_restrict_genes1, transcripts)
            if filter_remove_transcripts1: transcripts = filter( lambda x: x.mTranscript not in filter_remove_transcripts1, transcripts)            
        elif schema == genome2:
            if filter_restrict_genes2: transcripts = filter( lambda x: x.mGene in filter_restrict_genes2, transcripts)
            if filter_remove_transcripts2: transcripts = filter( lambda x: x.mTranscript not in filter_remove_transcripts2, transcripts)            
        else:
            continue

        if len(transcripts) == 0:
            neliminated += 1
            continue

        genes = GetGenes( transcripts )

        if schema == genome1:
            pairs.append( ( transcripts, {}, genes, {}, 0) )
        elif schema == genome2:
            pairs.append( ( {}, transcripts, {}, genes, 0) )
            
    return pairs


##########################################################################
def ClusterOrthologsByGenes( orthologs ):
    """cluster orthologs by genes.

    if an orthologous cluster contains the same genes in
    either species, they are merged.
    """

    map_gene2cluster = {}
    clusters = []

    for t1, t2, g1, g2, w in orthologs:

        ## get previous clusters
        pos = {}
        for g in g1.keys():
            k = "a%s" % g 
            if k in map_gene2cluster: 
                pos[map_gene2cluster[k]] = 1
        for g in g2.keys():
            k = "b%s" % g 
            if k in map_gene2cluster: 
                pos[map_gene2cluster[k]] = 1

        tt1 = t1
        tt2 = t2
        ww  = [w]
        ## add previous clusters to this cluster
        ## and clear previous clusters
        for p in pos:
            tt1 += clusters[p][0]
            tt2 += clusters[p][1]
            ww  += clusters[p][2]
            clusters[p] = None

        ## map previous clusters to new cluster
        n = len(clusters)
        for g in GetGenes( tt1 ):
            map_gene2cluster["a%s" % g] = n
        for g in GetGenes( tt2 ):
            map_gene2cluster["b%s" % g] = n

        ## append to clusters
        clusters.append( [tt1, tt2, ww] )            

    orthologs = []
    for c in clusters:
        if c:
            orthologs.append( (c[0], c[1], GetGenes(c[0]), GetGenes(c[1]), c[2]) )

    return orthologs
    

def GetDegeneracy( t1, t2 ):
    """get degeneracy of orthology assignments.

    given are two lists of transcripts.
    returns a tuple with gene and transcript degeneracy code, respectively.
    """
    g1 = GetGenes( t1 )
    g2 = GetGenes( t2 )        

    return "%i:%i" % (len(g1), len(g2)), "%i:%i" % ( len(t1), len(t2) )

    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
fasta2kmercontent.py
=============================================

:Author: Nick Ilott
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes as input a :term:`fasta` file from stdin and computes a k-nucleotide
content for each contig in the file. The output is a tab-delimited file of kmer counts:

     contig1  contig2  contig3  contig4  
n1
n2
n3


where n is the kmer and contig is the fasta entry.

The user specifies the kmer that is to be searched. Note that the longer the kmer, the
longer the script will take to run.


Usage
-----

Example::

   zcat in.fasta.gz | python fasta2kmercontent.py --kmer 4 > tetranucleotide_counts.tsv

In this example, for each contig in in.fasta.gz we count the occurrence of each four base
combination.


Alternative example::

   zcat in.fasta.gz | python fasta2kmercontent.py --kmer 4 --proportion > tetranucleotide_proportions.tsv

In this example, for each contig in in.fasta.gz we return the proportion of each four base
combination out of the total tetranucleotide occurences. --proportion overides the count
output. 


Type::

   python fasta2composition.py --help

for command line help.


Code
----

'''

import os
import sys
import re
import optparse
import CGAT.FastaIterator as FastaIterator
import itertools
import collections
import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = optparse.OptionParser( version = "%prog version: $Id: script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )
    parser.add_option("-k", "--kmer", dest="kmer", type="int",
                      help="supply kmer length")
    parser.add_option("-p", "--proportion", dest = "proportion", action="store_true",
                      help="output proportions - overides the default output")
    

    # add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    # do not allow greater than octonucleotide
    assert options.kmer <= 8, "cannot handle kmer of length %i" % options.kmer
    
    # how we deal with the nucleotides depends on the kmer length
    nucleotides = []
    for nucleotide in ["A", "C", "T", "G"]:
        nucleotides = nucleotides + [x for x in itertools.repeat(nucleotide, options.kmer)]
 
    E.info("retrieving %imer sequences" % options.kmer)
    # get all kmer sequences to query 
    kmers = set()
    for kmer in itertools.permutations(nucleotides, options.kmer):
        kmers.add(kmer)
        
    E.info("matching %imers in file" % options.kmer)
    # count the number of kmers in each sequence
    
    result = {}

    # NB assume that non fasta files are caught by FastaIterator
    total_entries = 0
    for fasta in FastaIterator.iterate(options.stdin):
        total_entries += 1
        result[fasta.title] = {}
        for kmer in kmers:
            counts = [m.start() for m in re.finditer("".join(kmer), fasta.sequence)]
            result[fasta.title][kmer] = len(counts) 
            
    E.info("writing results")
    # write out the results
    headers = result.keys()
    rows = set()
    for kmer_counts in result.values():
        for kmer, count in kmer_counts.iteritems():
            rows.add("".join(kmer))
    
    # write header row
    options.stdout.write("kmer\t" + "\t".join(headers) + "\n")

    # output proportions if required - normalises by 
    # sequence length
    E.info("computing total counts")
    totals = {}
    for header in headers:
        totals[header] = sum([result[header][tuple(row)] for row in rows])

    for row in rows:
        if options.proportion:
            options.stdout.write("\t".join([row] + [str(float(result[header][tuple(row)])/totals[header]) for header in headers]) + "\n")
        else:
            options.stdout.write("\t".join([row] + [str(result[header][tuple(row)]) for header in headers]) + "\n")

    E.info("written kmer counts for %i contigs" % total_entries)
    # write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
PredictionFile.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
import os, sys, string, tempfile
import PredictionParser

class PredictionFileIterator:
    
    def __init__( self, file ):
        self.mFile = file

    def __iter__( self ):
        return self
    
    def next( self ):

        while 1:
            line = self.mFile.readline()
            if not line: raise StopIteration
            if line[0] != "#": break
            
        entry = PredictionParser.PredictionParserEntry( expand = 0 )
        entry.Read(line)
        return entry
    
class PredictionFile:

    mMapKey2Field = {
        'mPredictionId' : (0, "n"),
        'mQueryToken' : (1, ""),
        'mSbjctToken' : (2, ""),
        'mSbjctStrand' : (3, ""),
        'score' : (6,"nr"),        
        'mQueryFrom' : (6,"n"),
        'mQueryTo' : (7,"n"),             
        'mSbjctGenomeFrom' : (22,"n"),
        'mSbjctGenomeTo' : (23,"n"),                        
        }

    def __init__( self ):
        self.mIsOpened = 0
    
    def __del__( self ):
        pass
    
    def open( self, filename = None, mode = "r"):
        if filename: self.mFilename = filename
        self.mFile = open( self.mFilename, mode )
        self.mIsOpened = 1

    def GetFileName( self ):
        return self.mFilename
    
    def close( self ):
        self.mIsOpened = 0
        self.mFile.close()
        
    def append( self, entry ):
        self.mFile.write( str(entry) + "\n" )
        
    def sort( self, fields, offset = 0 ):
        """sort file according to criteria."""

        if self.mIsOpened:
            reopen = 1
            self.close()
        else:
            reopen = 0
        
        sort_criteria = []
        for field in fields:
            id, modifier = self.mMapKey2Field[ field ]
            field_id = 1 + id + offset
            sort_criteria.append( "-k%i,%i%s" % (field_id, field_id, modifier) )            
            
        outfile, filename_temp = tempfile.mkstemp()
        os.close(outfile)

        ## empty fields are a problem with sort, which by default uses whitespace to non-whitespace
        ## transitions as field separate. 
        ## make \t explicit field separator.
        statement = "sort -t'\t' %s %s > %s" % (string.join( sort_criteria, " " ), self.mFilename, filename_temp)
        exit_code = os.system(statement)
        if exit_code:
            raise "error while sorting, statement =%s" % statement
        os.system( "mv %s %s" % (filename_temp, self.mFilename))

        if reopen: self.open()
        
    def __iter__( self ):
        self.mFile.seek(0)
        return PredictionFileIterator( self.mFile )

if __name__ == "__main__":

    file = PredictionFile()

    file.open( "tmp.predictions" )

    file.sort( ("mQueryToken", "mSbjctToken"), offset = -1 )
    
    for entry in file:
        print str(entry)

        
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
malis2malis.py - extract subsets of malis
=========================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

extract sequences from a set of multiple alignments and distribute into new directories.

Files can be split according to an input file given to the option --filename-components.
This file is a tab separated table containing the following fields::

   1       sequence        a sequence identifier
   2       input_id        The id under which the alignment is found.
   3       component_id    The component_id which under which the directory shall be stored.
                           This is optional, if no third column is specified, the input_id
                           is used.

Usage
-----

Example::

   python malis2malis.py --help

Type::

   python malis2malis.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time
import random
import types
import collections

USAGE="""python %s [OPTIONS]

""" % sys.argv[0]

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Mali as Mali
import CGAT.Intervals as Intervals

def addOptions( parser ):
    """add options to parser concerning components and masks."""

    parser.add_option("-b", "--pattern-component", dest="pattern_component", type="string",
                      help="how to extract identifier of mali from component name."  )

    parser.add_option("-c", "--filename-components", dest="filename_components", type="string",
                      help="filename of components to choose for each multiple alignment."  )

    parser.add_option("-e", "--pattern-filter", dest="pattern_filter", type="string",
                      help="pattern of how to extract filter from identifier."  )

    parser.add_option("-i", "--input-format", dest="input_format", type="choice",
                      choices=("plain", "fasta", "clustal", "phylip" ),
                      help="input format of multiple alignment"  )

    parser.add_option("-o", "--output-format", dest="output_format", type="choice",
                      choices=( "fasta", "codeml", "phylip", "profile", "stockholm" ),
                      help="output format of multiple alignment"  )

    parser.add_option("--filename-mask-regions", dest="filename_mask_regions", type="string",
                      help="""mask regions given file. Input format is component_id\tstart\tend with coordinates in
the multiple alignment.""" )

    parser.add_option("--filename-extract-regions", dest="filename_extract_regions", type="string",
                      help="""extract regions given file. Input format is component_id\tstart\tend with coordinates in
the multiple alignment. The alignment is restricted to the given coordinates by masking everything else.""" )

    parser.add_option("--filename-annotate-regions", dest="filename_annotate_regions", type="string",
                      help="""annotate regions given file. Input format is component_id\tstart\tend\tlabel with coordinates in
the multiple alignment. The label should be a single letter. If the file is empty, the full alignment is output with label N.""")

    parser.add_option( "--use-input-id", dest="use_input_id", action="store_true",
                       help="""use input-id instead of component_id in --filename-X-regions""" )

    parser.add_option("-a", "--pattern-mali", dest="pattern_mali", type="string",
                      help="filename pattern for input multiple alignment files."  )
    
    parser.add_option("--test", dest="test", type = "int",
                      help="test output with first # components."  )

    parser.add_option("--skip-doubles", dest="skip_doubles", action="store_true",
                      help="skip doubles, takes first entry encountered."  )

    parser.add_option("--ignore-missing", dest="ignore_missing", action="store_true",
                      help="ignore missing alignments [%default]."  )

    parser.add_option("--remove-all-gaps", dest = "remove_all_gaps", type = int,
                      help="remove fully masked or gapped positions in a multiple alignment. The integer parameter supplies the frame. Use 3 for codon alignments." )

    parser.add_option("--remove-any-gaps", dest = "remove_any_gaps", type = int,
                      help="remove positions in a multiple alignment containing at least one gap. The integer parameter supplies the frame. Use 3 for codon alignments." )

    parser.add_option("--sample", dest="sample", type = "int",
                      help="sample # components."  )
    
    parser.add_option("--sample-method", dest="sample_method", type="choice",
                      choices=("simple-without-replacement", "length-without-replacement", ),
                      help="sampling method: without-replacement=sample without replacement. length-sample: sample components of same length (without replacement).")

    parser.add_option("--filename-sample-reference", dest="filename_sample_reference", type="string",
                      help="filename to with reference components. The components should be part of the file supplied by --filename-components." )

    parser.add_option("--minimum-mali-length", dest="minimum_mali_length", type="int",
                      help="minimum multiple alignment length." )

    parser.set_defaults(
        pattern_mali = None,
        pattern_component="(^\S+)",
        pattern_filter="(^\S+)",
        input_format="fasta",
        output_format="fasta",
        test = 0,
        skip_doubles = False,
        remove_all_gaps = None,
        remove_any_gaps = None,
        sample = 0,
        filename_sample_reference = None,
        sample_method = "simple-without-replacement",
        filename_components = None,
        filename_mask_regions = None,
        filename_annotated_regions = None,
        filename_extract_regions = None,
        minimum_mali_length = 0,
        use_input_id = False,
        ignore_missing = False,
        )

def readComponents( options ):
    """read components from filename supplied in the options.
    """
    if options.filename_components:
        map_seq_id2component =\
                             IOTools.ReadMap( open(options.filename_components, "r"),
                                              columns = "all",
                                              both_directions = False)

        map_component2seq_id = {}
        map_component2input_id = {}
        for key, val in map_seq_id2component.items():
            if type(val) == types.StringType:
                input_id = val
                output_id = val
            elif type(val) == types.TupleType:
                if len(val) == 2:
                    input_id = val[0]
                    output_id = val[1]
                else:
                    input_id = val[0]
                    output_id = val[0]
            else:
                raise ValueError("error in reading %s: %s->%s" % (options.filename_components, key, val))
            
            if output_id not in map_component2seq_id: map_component2seq_id[output_id] = []
            map_component2seq_id[output_id].append(key)
            map_component2input_id[output_id] = input_id
        return map_seq_id2component, map_component2seq_id, map_component2input_id
    else:
        return None, None, None

def readMasks( options, map_component2input_id ):
    """read masking information from filename supplied in the options.
    """

    if options.use_input_id:
        map_id2component = IOTools.getInvertedDictionary( map_component2input_id )
    else:
        map_id2component = {}
        for component in map_component2input_id.keys():
            map_id2component[component] = (component,)

    if options.filename_mask_regions:
        map_component2masks = collections.defaultdict( list )
        if not os.path.exists( options.filename_mask_regions ):
            options.stdlog.write("# could not find %s - ignored \n" % options.filename_mask_regions )
        else:
            for line in open(options.filename_mask_regions, "r" ):
                if line[0] == "#": continue
                id, start, end = line[:-1].split("\t")
                try:
                    start, end = int(start), int(end)
                except ValueError:
                    continue
                if id not in map_id2component: continue
                for x in map_id2component[id]:
                    map_component2masks[x].append( (start, end) )
            if options.loglevel >= 1:
                options.stdlog.write("# read masks for %i malis.\n" % len(map_component2masks))
                options.stdlog.flush()
    else:
        map_component2masks = None

    return map_component2masks

def readExtracts( options, map_component2input_id ):
    """read extract information from filename supplied in the options.
    """
    if not options.filename_extract_regions: return None

    if options.use_input_id:
        map_id2component = IOTools.getInvertedDictionary( map_component2input_id )
    else:
        map_id2component = {}
        for component in map_component2input_id.keys():
            map_id2component[component] = (component,)

    map_component2extracts = collections.defaultdict( list )
    if not os.path.exists( options.filename_extract_regions ):
        options.stdlog.write("# could not find %s - ignored \n" % options.filename_extract_regions )
    else:
        for line in open(options.filename_extract_regions, "r" ):
            if line[0] == "#": continue
            id, start, end = line[:-1].split("\t")
            start, end = int(start), int(end)
            if id not in map_id2component: continue
            for x in map_id2component[id]:
                map_component2extracts[x].append( (start, end) )
        if options.loglevel >= 1:
            options.stdlog.write("# read extracts for %i malis.\n" % len(map_component2extracts))
            options.stdlog.flush()

    return map_component2extracts

def readAnnotations( options, map_component2input_id ):
    """read annotation information from filename supplied in the options.
    """
    if not options.filename_annotate_regions: return None

    if options.use_input_id:
        map_id2component = IOTools.getInvertedDictionary( map_component2input_id )
    else:
        map_id2component = {}
        for component in map_component2input_id.keys():
            map_id2component[component] = (component,)

    map_component2annotations = collections.defaultdict( list )
    if not os.path.exists( options.filename_annotate_regions ):
        options.stdlog.write("# could not find %s - ignored \n" % options.filename_annotate_regions )
    else:
        for line in open(options.filename_annotate_regions, "r" ):
            if line[0] == "#": continue
            try:
                id, start, end, label = line[:-1].split("\t")
            except ValueError:
                raise ValueError("parsing error in line %s\n" % (line[:-1]))

            start, end = int(start), int(end)
            if id not in map_id2component: continue
            for x in map_id2component[id]:
                map_component2annotations[x].append( (start, end, label ) )

        if options.loglevel >= 1:
            options.stdlog.write("# read annotations for %i malis.\n" % len(map_component2annotations))
            options.stdlog.flush()

    return map_component2annotations

def maskAlignment( mali, 
                   map_component2masks, 
                   map_component2extracts,
                   map_sample2reference,
                   options ):
    """mask an alignment.

    If map_sample2reference is given, coordinates in references are used to
    mask residues in sample.
    """

    id = mali.getName()

    if options.loglevel >= 5:
        options.stdout.write( "# multiple alignment %s before masking:\n" % id )
        mali.writeToFile( sys.stdout )

    def getMasks( id, map1, map_sample2reference):
        if map_sample2reference and id in map_sample2reference:
            xid = map_sample2reference[id]
            if options.loglevel >= 1:
                options.stdlog.write( "# using mapped coordinates from %s for %s\n" % (xid, id))
        else:
            xid = id
        if xid in map1:
            return map1[xid]
        else:
            return []

    ############################################################################
    ## mask alignment
    if map_component2masks:
        masks = getMasks( id, map_component2masks, map_sample2reference )
        if masks:
            for start, end in masks:
                
                if options.loglevel >= 1:
                    options.stdlog.write("# component: %s: masking region due to mask %i-%i\n" % (id, start, end))

                mali.maskColumns( range(start, min(end, mali.getWidth() ) ) )

    ############################################################################
    ## extract regions from an alignment by masking everything else
    if map_component2extracts:
        masks = getMasks( id, map_component2extracts, map_sample2reference )

        if masks:
            other_masks = Intervals.complementIntervals( masks, 0, mali.getWidth() )
            for start, end in other_masks:
                if options.loglevel >= 1:
                    options.stdlog.write("# component: %s: masking region due to extract %i-%i\n" % (id, start, end))

                mali.maskColumns( range(start, min(end, mali.getWidth()) ))

    if options.loglevel >= 5:
        options.stdout.write( "# multiple alignment after masking:\n" )
        mali.writeToFile( sys.stdout )

    if  mali.getAlphabet() == "aa":
        gap_chars = "Xx-"
    else:
        gap_chars = "XxNn-"

    if options.remove_all_gaps:
        
        width_before = mali.getNumColumns()

        mali.removePattern(
            match_function = lambda x: x in gap_chars,
            minimum_matches = mali.getNumSequences(),
            search_frame = 1,
            delete_frame = options.remove_all_gaps )

        width_after = mali.getNumColumns()

        if options.loglevel >= 1:
            options.stdlog.write("# component: %s: removed %i fully gapped/masked columns, old size=%i, new size=%i\n" % (id, width_before-width_after, width_before, width_after))
            
    if options.remove_any_gaps:
        
        width_before = mali.getNumColumns()

        mali.removePattern(
            match_function = lambda x: x in gap_chars,
            minimum_matches = 1,
            search_frame = 1,
            delete_frame = options.remove_any_gaps )

        width_after = mali.getNumColumns()

        if options.loglevel >= 1:
            options.stdlog.write("# component: %s: removed %i columns containing at least one gap/mask, old size=%i, new size=%i\n" % (id, width_before-width_after, width_before, width_after))


    if options.loglevel >= 5:
        options.stdout.write( "# multiple alignment after cleaning:\n" )
        mali.writeToFile( sys.stdout )

def annotateAlignment( mali,
                       map_component2annotations, 
                       options ):
    """annotate an alignment.

    Do not use this function if options.clean_mali is True and 
    regions have been extracted. Both might result in a change
    of the length of the multiple alignment and thus the annotated
    coordinates would be incorrect.
    """

    id = mali.getName()

    if map_component2annotations == None: return

    max_length = mali.getNumColumns()

    if len(map_component2annotations) == 0:
        mali.addAnnotation( "STATE", "N" * max_length )
        return

    if id not in map_component2annotations:

        if options.loglevel >= 1:
            options.stdlog.write("# no annotations found for %s.\n" % id )
        return

    max_length = mali.getNumColumns()
    annotation = [ "-" ] * max_length
    
    for start, end, label in map_component2annotations[id]:
        end = min(end, max_length )
        annotation[start:end] = [ label ] * (end-start)
        
    mali.addAnnotation( "STATE", "".join(annotation) )

master_mali = None

def getMali( component_id, 
             map_component2seq_id, 
             map_component2input_id,
             id_filter,
             options ):
    
    global master_mali

    rx_component = re.compile(options.pattern_component)    

    mali = Mali.Mali()

    nsubstitutions=len(re.findall("%s", options.pattern_mali))

    input_id = rx_component.search( component_id ).groups()[0]
    input_id = map_component2input_id[input_id]

    if nsubstitutions == 0:
        
        if master_mali == None:

            master_mali = Mali.Mali()

            E.debug( "retrieving multiple alignment from file %s" % (options.pattern_mali) )

            master_mali.readFromFile( open( options.pattern_mali, "r"), format=options.input_format )

        for s in map_component2seq_id[component_id]:

            if options.pattern_filter and id_filter:
                f = re.search(options.pattern_filter, s).groups()[0]

                if f not in id_filter:
                    E.debug( "removing %s from %s: not in filter" % (f, component_id))
                    continue

            if options.output_format == "codeml":
                if len(master_mali[s]) % 3 != 0:
                    raise ValueError("length of sequence %s is not a multiple of 3: %i" % (s, len(master_mali[s])))

            if s in mali:
                if options.skip_doubles:
                    E.warn( "skipped double entry %s in component %s" % (s, component_id ) )
                    return None
                else:
                    raise ValueError("duplicate entry %s in component %s" % (s, component_id))

            mali.addEntry( master_mali.getEntry(s) )
            
    else:

        input_filename = options.pattern_mali % tuple([input_id] * nsubstitutions)

        E.debug( "retrieving multiple alignment for component %s from file %s" % (component_id, input_filename))

        if not os.path.exists( input_filename):
            if options.ignore_missing:
                E.warn( "alignment %s not found" % input_filename )
                return None
            else:
                raise OSError( "alignment %s not found" % input_filename )

        mali.readFromFile( open( input_filename, "r"), format=options.input_format )

        ## get identifiers (and make a copy)
        s = tuple(mali.getIdentifiers())
        for ss in s:
            if options.pattern_filter and id_filter:
                f = re.search(options.pattern_filter, ss).groups()[0]

                if f not in id_filter:
                    mali.deleteEntry( ss )
                    if options.loglevel >= 5:
                        options.stdlog.write( "# removing %s from %s: not in filter.\n" % (ss, component_id))
                    continue

                
            if ss not in map_component2seq_id[component_id]:
                if options.loglevel >= 5:
                    options.stdlog.write( "# removing %s from %s: not in component list.\n" % (ss, component_id))
                mali.deleteEntry( ss )
            else:
                if options.output_format == "codeml":
                    if len(mali[ss]) % 3 != 0:
                        raise "length of sequence %s is not a multiple of 3: %i" % (ss, len(mali[ss]))
        
    mali.setName( component_id )

    return mali

def selectComponents( component_ids,                            
                      map_component2seq_id, 
                      map_component2input_id,
                      id_filter,
                      options ):

    """select a set of components from component_ids.
    """

    map_sample2reference = {}

    if options.sample:

        if options.sample_method == "simple-without-replacement":
            random.shuffle( component_ids )
            
        elif options.sample_method == "length-without-replacement":
            
            map_component_id2length = {}

            for component_id in component_ids:

                mali = getMali( component_id, 
                                map_component2seq_id, 
                                map_component2input_id,
                                id_filter,
                                options )

                if not mali: continue

                map_component_id2length[component_id] = mali.getWidth()
                
            reference_ids, nerrors = IOTools.ReadList( open( options.filename_sample_reference, "r") )
            
            # do not sample from the reference set
            sampled_components = set( reference_ids )

            new_component_ids = []

            ninput, noutput, nskipped = 0, 0, 0
            # now go through reference set
            for ref_id in reference_ids:

                ninput += 1
                if ref_id not in map_component_id2length:
                    if options.loglevel >= 1:
                        options.stdlog.write("# reference component %s not found.\n" % (str( ref_id ) ))
                    nskipped += 1
                    continue
                
                ref_length = map_component_id2length[ref_id]
                ref_length_min = ref_length - 50
                ref_length_max = ref_length + 50

                # find all components with a length similar to ref_length excluding previously sampled ones.
                test_components = filter( lambda x: ref_length_min < map_component_id2length[x] < ref_length_max, component_ids )
                test_components = list(set( test_components ).difference( sampled_components ))
                
                if len(test_components) == 0:
                    if options.loglevel >= 1:
                        options.stdlog.write("# reference components %s: skipped - no others with equivalent length around %i found." % \
                                                 ( ref_id, ref_length ) )
                    nskipped += 1
                    continue

                random.shuffle( test_components )
                
                component_id = test_components[0]
                sampled_components.add( component_id )

                map_sample2reference[ component_id] = ref_id

                if options.loglevel >= 1:
                    options.stdlog.write("# reference component mapping: %s\t%s\t%i\t%i\t%i\n" % ( 
                            ref_id, 
                            component_id, 
                            ref_length,
                            map_component_id2length[component_id],
                            len(test_components) ) )
                                         
                new_component_ids.append( component_id )
                noutput += 1

            if options.loglevel >= 1:
                options.stdlog.write("# sampling results: ninput=%i, noutput=%i, nskipped=%i\n" % (ninput, noutput, nskipped))
                    
            component_ids = new_component_ids
            options.sample = len( new_component_ids )

    return component_ids, map_sample2reference


##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: malis2malis.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-d", "--pattern-output", dest="pattern_output", type="string",
                      help="filename pattern for output multiple alignment files."  )

    parser.add_option("-f", "--filename-filter", dest="filename_filter", type="string",
                      help="filename with strings to filter by."  )

    parser.add_option( "--list-filter", dest="list_filter", type="string",
                      help="list of strings to filter by."  )

    parser.set_defaults(
        pattern_output="%s.mali",
        methods = "",
        parameters = "",
        filename_filter = None,
        list_filter = None,
        )

    addOptions( parser )

    (options, args) = E.Start( parser )

    options.methods = options.methods.split(",")
    options.parameters = options.parameters.split(",")    

    if not options.pattern_mali:
        raise "Please specifiy a pattern to find the malis using --pattern-mali"

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read components
    ####################################################################
    map_seq_id2component, map_component2seq_id, map_component2input_id = \
        readComponents( options )

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read filtering information
    ####################################################################
    if options.filename_filter:
        id_filter, nerrors = IOTools.ReadList( open(options.filename_filter, "r") )
        if options.loglevel >= 1:
            options.stdlog.write("# read %i identifiers to filter each multiple alignment with.\n" % len(id_filter))
            options.stdlog.flush()
    elif options.list_filter:
        id_filter = options.list_filter.split(",")
    else:
        id_filter = None

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read regions to mask
    ####################################################################
    map_component2masks = readMasks( options, map_component2input_id )

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read regions to extract
    ####################################################################
    map_component2extracts = readExtracts( options, map_component2input_id)

    ####################################################################
    ####################################################################
    ####################################################################
    ## Read regions to annotate
    ####################################################################
    map_component2annotations = readAnnotations( options, map_component2input_id)

    ####################################################################
    ####################################################################
    ####################################################################
    ## Prepare for run
    ####################################################################
    component_ids = map_component2seq_id.keys()
    component_ids.sort()
    
    if options.loglevel >= 1:
        options.stdlog.write("# %i component ids to start with.\n" % (len(component_ids )))

    component_ids, map_sample2reference = selectComponents( component_ids,
                                      map_component2seq_id,
                                      map_component2input_id,
                                      id_filter,
                                      options )

    if options.test:
        component_ids = component_ids[:options.test]

    if options.loglevel >= 1:
        options.stdlog.write("# %i component ids selected for output.\n" % (len(component_ids )))
        
    ninput = 0
    noutput = 0
    nskipped = 0
    nskipped_length = 0

    for component_id in component_ids:

        ninput += 1

        if options.loglevel >= 3:
            options.stdlog.write("# processing component %s\n" % (component_id) )
        
        mali = getMali( component_id, 
                        map_component2seq_id,
                        map_component2input_id,
                        id_filter,
                        options )
        
        if mali == None:
            E.warn ("empty mali returned for component %s" % (component_id) )
            nskipped += 1
            continue

        if mali.getNumColumns() == 0:
            E.warn( "skipping output of empty alignment for component %s" % (component_id) )
            nskipped += 1
            continue

        mali.setName( str(component_id) )

        ###############################################################
        ## add annotations
        if map_component2annotations != None:
            annotateAlignment( mali, 
                               map_component2annotations,
                               options )
            

        ###############################################################
        ## mask the alignment
        maskAlignment( mali, 
                       map_component2masks,
                       map_component2extracts, 
                       map_sample2reference,
                       options )

        if mali.getNumColumns() < options.minimum_mali_length:
            nskipped_length += 1
            if options.loglevel >= 1:
                options.stdlog.write("# component %s: skipped, because length %i less than threshold.\n" % (component_id, mali.getNumColumns()))
            continue

        ###############################################################
        ## prepare the mali for output
        if "%s" not in options.pattern_output:
            append = True
        else:
            append = False

        output_filename = re.sub( "%s", component_id, options.pattern_output )
        input_id = map_component2input_id[component_id]

        if options.loglevel >= 2:
            options.stdlog.write("# component %s: input from %s, goes to %s\n" % (component_id, input_id, output_filename) )

        dirname = os.path.dirname(output_filename)
            
        if dirname and not os.path.exists( dirname ):
            os.makedirs( dirname )

        if not os.path.exists( output_filename ):
            mali.writeToFile( open( output_filename, "w"), format=options.output_format )
            noutput += 1
        else:
            if append:
                mali.writeToFile( open( output_filename, "a"), format=options.output_format )
                noutput += 1
            else:
                if options.loglevel >= 1:
                    options.stdlog.write("# skipping because output for component %s already exists: %s\n" % (component_id, output_filename))                        
                nskipped += 1

        # if we only sample, stop if you have reached
        # the desired number
        if options.sample and noutput == options.sample:
            break

    E.info("ninput=%i, noutput=%i, nskipped=%i, nskipped_length=%i" % (ninput, noutput, nskipped, nskipped_length))
        
    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
sequence2alignment.py - convert aligned sequences to an alignment
=================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

map residue positions in sequences with gaps to positions in
the aligned string.

This script is useful to map aligned strings to their multiple
alignment positions.

Usage
-----

Example::

   python sequence2alignment.py --help

Type::

   python sequence2alignment.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import math
import time
import random
import types

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import alignlib
import CGAT.FastaIterator as FastaIterator

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: sequence2alignment.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.set_defaults(
        )

    (options, args) = E.Start( parser )

    iterator = FastaIterator.FastaIterator( sys.stdin )

    ninput, noutput, nskipped = 0, 0, 0
    
    options.stdout.write( "query\tsbjct\tquery_from\tquery_to\tsbjct_from\tsbjct_to\tquery_starts\tsbjct_starts\tblock_sizes\n" )

    while 1:
        try:
            cur_record = iterator.next()
        except StopIteration:
            break

        ninput += 1
        
        sequence = re.sub( " ", "", cur_record.sequence)
        l = len(sequence)

        map_sequence2mali = alignlib.makeAlignmentVector()        

        alignlib.AlignmentFormatExplicit( 0, sequence,
                                          0, "X" * l ).copy( map_sequence2mali )

        options.stdout.write( "\t".join( (
                cur_record.title,
                "ref",
                str( alignlib.AlignmentFormatBlocks( map_sequence2mali ) ) ) ) + "\n" )
        
        noutput += 1

    if options.loglevel >= 1:
        options.stdlog.write("# ninput=%i, noutput=%i, nskipped=%i.\n" % (ninput, noutput, nskipped))
        
    E.Stop()
    
